import base64
import binascii

from Crypto.Util.Padding import pad
from Crypto.Cipher import AES


class BaseEncode(object):

    def __init__(self):
        pass

    @classmethod
    def get_byte_md5_value(cls, bytes):
        """
        获得byte md5值
        :param bytes:需要操作的二进制
        :return:
        """
        import hashlib
        myMd5 = hashlib.md5(bytes)
        myMd5_Digest = myMd5.hexdigest()
        return myMd5_Digest

    @classmethod
    def get_md5_value(cls, src):
        """
        获得字符串md5值
        :param src:需要操作的字符串
        :return:
        """
        import hashlib
        myMd5 = hashlib.md5()
        myMd5.update(src.encode("utf8"))
        myMd5_Digest = myMd5.hexdigest()
        return myMd5_Digest

    @classmethod
    def get_md5_value_16bit(cls, src):
        """
        获取16位md5
        :param src:
        :return:
        """
        return cls.get_md5_value(src)[8:-8]

    @classmethod
    def get_byte_md5_value_16bit(cls, bytes):
        """
        获取16位md5
        :param bytes:
        :return:
        """
        return cls.get_byte_md5_value(bytes)[8:-8]

    @classmethod
    def get_sha1_value(cls, src):
        """
         获得字符串sha1值
        :param src:
        :return:
        """
        import hashlib
        mySha1 = hashlib.sha1()
        mySha1.update(src)
        mySha1_Digest = mySha1.hexdigest()
        return mySha1_Digest

    @classmethod
    def get_base64(cls, src):
        """
        输入字符串，编码成base64
        :param src:
        :return:
        """
        strEncode = base64.b64encode(src.encode('utf8')).decode('utf8')
        return strEncode

    @classmethod
    def base64_get_str(cls, base64_str):
        """
        输入base64字符串，输出原始字符串
        :param base64_str:
        :return:
        """
        src = base64.b64decode(base64_str.encode('utf8')).decode('utf8')
        return src

    @classmethod
    def aes_encode(cls, password, text, mode, iv=None):
        """
         https://blog.csdn.net/qq_42334096/article/details/122847876
        在AES标准规范中，分组长度只能是128位 每个分组为16个字节(每个字节8位)。
        密钥长度可以使用128位、192位或256位。密钥的长度不同，推荐加密轮数也不同。
        AES	密钥长度(32位比特字)	分组长度(32位比特字)	加密轮数
        AES-128	4	4	10
        AES-192	6	4	11
        AES-256	8	4	14
        填充模式：这是因为如果明文不是128位(16字节)的则需要进行填充，需要将明文补充到16个字节整数倍的长度。在我们进行加解密时需要采用同样的填充方式，
        否则无法解密成功。填充模式有：No Padding、PKCS5 Padding、PKCS7 Padding、ISO10126 Padding、Ansix923 Padding、Zero Padding等等。
        pip install pycryptodome
        （亲测，目前不用改文件夹名字了） 但是，在使用的时候导包是有问题的，这个时候只要修改一个文件夹的名称就可以完美解决这个问题
        C:\用户\Administrator\AppData\Local\Programs\Python\Python36\Lib\site-packages

        找到这个路径，下面有一个文件夹叫做crypto,将c改成C，对就是改成大写就ok了！
        liunx: pip install pycrypto
        :param password:
        :param text:
        :param mode:
        :return:
        """
        if mode == AES.MODE_ECB:
            assert len(password) % 16 == 0, Exception("密匙 位数应该为16的倍数")
            assert len(text) % 16 == 0, Exception("加密文本 位数应该为16的倍数")
            # MODE_ECB 秘钥必须为16字节或者16字节的倍数的字节型数据。
            # MODE_ECB 明文必须为16字节或者16字节的倍数的字节型数据，如果不够16字节需要进行补全，关于补全规则，后面会在补全模式中具体介绍。
            aes = AES.new(password, AES.MODE_ECB)  # 创建一个aes对象
            # AES.MODE_ECB 表示模式是ECB模式
            en_text = aes.encrypt(text)  # 加密明文
            return en_text
        if mode == AES.MODE_CBC:
            # 1. 在Python中进行AES加密解密时，所传入的密文、明文、秘钥、iv偏移量、都需要是bytes（字节型）数据。python 在构建aes对象时也只能接受bytes类型数据。
            # 2.当秘钥，iv偏移量，待加密的明文，字节长度不够16字节或者16字节倍数的时候需要进行补全。
            # 3. CBC模式需要重新生成AES对象，为了防止这类错误，我写代码无论是什么模式都重新生成AES对象。
            assert iv is not None
            aes = AES.new(password, AES.MODE_CBC, iv)  # 创建一个aes对象
            # AES.MODE_CBC 表示模式是CBC模式
            en_text = aes.encrypt(text)
            return en_text

    @classmethod
    def aes_decode(cls, password, en_text, mode, iv=None):
        """
        :return:
        """
        if mode == AES.MODE_ECB:
            assert len(password) % 16 == 0, Exception("密匙 位数应该为16的倍数")
            assert len(en_text) % 16 == 0, Exception("密文 位数应该为16的倍数")
            aes = AES.new(password, AES.MODE_ECB)  # 创建一个aes对象
            # AES.MODE_ECB 表示模式是ECB模式
            text = aes.decrypt(en_text)  # 加密明文
            return text
        if mode == AES.MODE_CBC:
            assert iv is not None
            aes = AES.new(password, AES.MODE_CBC, iv)  # 创建一个aes对象
            # AES.MODE_CBC 表示模式是CBC模式
            text = aes.decrypt(en_text)
            return text


# 数据类
class MData():
    def __init__(self, data=b"", characterSet='utf-8'):
        # data肯定为bytes
        self.data = data
        self.characterSet = characterSet

    def saveData(self, FileName):
        with open(FileName, 'wb') as f:
            f.write(self.data)

    def fromString(self, data):
        self.data = data.encode(self.characterSet)
        return self.data

    def fromBase64(self, data):
        self.data = base64.b64decode(data.encode(self.characterSet))
        return self.data

    def fromHexStr(self, data):
        self.data = binascii.a2b_hex(data)
        return self.data

    def toString(self):
        return self.data.decode(self.characterSet)

    def toBase64(self):
        return base64.b64encode(self.data).decode()

    def toHexStr(self):
        return binascii.b2a_hex(self.data).decode()

    def toBytes(self):
        return self.data

    def __str__(self):
        try:
            return self.toString()
        except Exception:
            return self.toBase64()


### 封装类
class AEScryptor():
    def __init__(self, key, mode, iv='', paddingMode="NoPadding", characterSet="utf-8"):
        '''
        构建一个AES对象
        key: 秘钥，字节型数据
        mode: 使用模式，只提供两种，AES.MODE_CBC, AES.MODE_ECB
        iv： iv偏移量，字节型数据
        paddingMode: 填充模式，默认为NoPadding, 可选NoPadding，ZeroPadding，PKCS5Padding，PKCS7Padding
        characterSet: 字符集编码
        '''
        self.key = key
        self.mode = mode
        self.iv = iv
        self.characterSet = characterSet
        self.paddingMode = paddingMode
        self.data = ""

    def __ZeroPadding(self, data):
        data += b'\x00'
        while len(data) % 16 != 0:
            data += b'\x00'
        return data

    def __StripZeroPadding(self, data):
        data = data[:-1]
        while len(data) % 16 != 0:
            data = data.rstrip(b'\x00')
            if data[-1] != b"\x00":
                break
        return data

    def __PKCS5_7Padding(self, data):
        needSize = 16 - len(data) % 16
        if needSize == 0:
            needSize = 16
        return data + needSize.to_bytes(1, 'little') * needSize

    def __StripPKCS5_7Padding(self, data):
        paddingSize = data[-1]
        return data.rstrip(paddingSize.to_bytes(1, 'little'))

    def __paddingData(self, data):
        if self.paddingMode == "NoPadding":
            if len(data) % 16 == 0:
                return data
            else:
                return self.__ZeroPadding(data)
        elif self.paddingMode == "ZeroPadding":
            return self.__ZeroPadding(data)
        elif self.paddingMode == "PKCS5Padding" or self.paddingMode == "PKCS7Padding":
            return self.__PKCS5_7Padding(data)
        else:
            print("不支持Padding")

    def __stripPaddingData(self, data):
        if self.paddingMode == "NoPadding":
            return self.__StripZeroPadding(data)
        elif self.paddingMode == "ZeroPadding":
            return self.__StripZeroPadding(data)

        elif self.paddingMode == "PKCS5Padding" or self.paddingMode == "PKCS7Padding":
            return self.__StripPKCS5_7Padding(data)
        else:
            print("不支持Padding")

    def setCharacterSet(self, characterSet):
        '''
        设置字符集编码
        characterSet: 字符集编码
        '''
        self.characterSet = characterSet

    def setPaddingMode(self, mode):
        '''
        设置填充模式
        mode: 可选NoPadding，ZeroPadding，PKCS5Padding，PKCS7Padding
        '''
        self.paddingMode = mode

    def decryptFromBase64(self, entext):
        '''
        从base64编码字符串编码进行AES解密
        entext: 数据类型str
        '''
        mData = MData(characterSet=self.characterSet)
        self.data = mData.fromBase64(entext)
        return self.__decrypt()

    def decryptFromHexStr(self, entext):
        '''
        从hexstr编码字符串编码进行AES解密
        entext: 数据类型str
        '''
        mData = MData(characterSet=self.characterSet)
        self.data = mData.fromHexStr(entext)
        return self.__decrypt()

    def decryptFromString(self, entext):
        '''
        从字符串进行AES解密
        entext: 数据类型str
        '''
        mData = MData(characterSet=self.characterSet)
        self.data = mData.fromString(entext)
        return self.__decrypt()

    def decryptFromBytes(self, entext):
        '''
        从二进制进行AES解密
        entext: 数据类型bytes
        '''
        self.data = entext
        return self.__decrypt()

    def encryptFromString(self, data):
        '''
        对字符串进行AES加密
        data: 待加密字符串，数据类型为str
        '''
        self.data = data.encode(self.characterSet)
        return self.__encrypt()

    def __encrypt(self):
        if self.mode == AES.MODE_CBC:
            aes = AES.new(self.key, self.mode, self.iv)
        elif self.mode == AES.MODE_ECB:
            aes = AES.new(self.key, self.mode)
        else:
            print("不支持这种模式")
            return

        data = self.__paddingData(self.data)
        enData = aes.encrypt(data)
        return MData(enData)

    def __decrypt(self):
        if self.mode == AES.MODE_CBC:
            aes = AES.new(self.key, self.mode, self.iv)
        elif self.mode == AES.MODE_ECB:
            aes = AES.new(self.key, self.mode)
        else:
            print("不支持这种模式")
            return
        data = aes.decrypt(self.data)
        mData = MData(self.__stripPaddingData(data), characterSet=self.characterSet)
        return mData


# if __name__ == '__main__':
#     key = b"1234567812345678"
#     iv = b"0000000000000000"
#     aes = AEScryptor(key, AES.MODE_CBC, iv, paddingMode="ZeroPadding", characterSet='utf-8')
#
#     data = "好好学习"
#     rData = aes.encryptFromString(data)
#     print("密文：", rData.toBase64())
#     rData = aes.decryptFromBase64(rData.toBase64())
#     print("明文：", rData)



import click

###########################################
# 同项目调用基础包
import datetime
import gzip
import json
import os
import sys
import time

filepath = os.path.abspath(__file__)
pathlist = filepath.split(os.sep)
pathlist = pathlist[:-5]
TopPath = os.sep.join(pathlist)
sys.path.insert(0, TopPath)
print(TopPath)
############################################


from re_common.baselibrary.tools.move_mongo.move_mongo_table import MoveMongodbColl, MoveMongodbThreadRun


class Configs(object):

    def __init__(self):
        self.db3_path = r"D:\config\mvmg\db3\test_images.db3"
        self.db3_encoding = "utf-8"
        self.mgdb_conn = "mongodb://192.168.31.30:32417/"
        self.mgdb_conn_motor = "mongodb://192.168.31.30:32417/htmljson.wanfang_ref?authSource=htmljson"
        self.mgdb_db = "htmljson"
        self.mgdb_col = "cx_journal_detail"

        self.mgdb_conn2_motor = "mongodb://cjrw:vipdatacenter@192.168.31.243:32920,192.168.31.206:32920,192.168.31.208:32920/?authSource=htmljson"
        self.mgdb_db2 = "htmljson"
        self.mgdb_col2 = "cx_journal_detail"

        self.mq_name = "mongodb.move.send"
        self.mq_name_work = "mongodb.move.worker"

        self.error_dir = r"D:\config\mvmg\log"


conf = Configs()


def one_init():
    # 第一步 初始化数据到db3文件
    MoveMongodbColl(conf).one_init()


# 第二步 开始转移 使用send_work_recv结构 共开三个进程

def send():
    MoveMongodbColl(conf).two_send()


def works():
    MoveMongodbThreadRun(1, conf).run()


def works_many():
    mmtr = MoveMongodbThreadRun(1, conf)
    mmtr.is_many_move = True
    mmtr.num_list = 1000
    mmtr.num_info_list = 10
    mmtr.run()


def recv():
    MoveMongodbColl(conf).two_recv()


@click.command()
@click.option('--name',
              help='func name')
def main(name):
    if name == "send":
        send()
    elif name == "one_init":
        one_init()
    elif name == "works":
        works()

    elif name == "works_many":
        works_many()

    elif name == "recv":
        recv()


if __name__ == '__main__':
    main()



# -*- coding:utf-8 -*-
# @Time : 2021/12/2 9:38
# @Author: suhong
# @File : __init__.py.py
# @Function :

from re_common.baselibrary.utils.basetime import BaseTime


x = BaseTime().get_beijin_date_strins("%Y%m%d") + "00"
print(x)


import asyncio

from pathlib import Path
import time
from typing import Any, Awaitable, Callable
from re_common.v2.baselibrary.tools.concurrency import AsyncTaskPool
from re_common.v2.baselibrary.tools.hdfs_base_processor import HDFSBaseProcessor


class HDFSLineProcessor(HDFSBaseProcessor):
    async def _process_data(self, data, process_func, pool):
        """处理数据并执行处理函数"""
        retry_count = 0
        while retry_count < self.retries:
            try:
                return await process_func(data, pool)  # 成功处理后退出
            except Exception as e:
                retry_count += 1
                print(f"处理数据时发生错误: {e}, 正在重试 {retry_count}/{self.retries}, data: {data}")
                await asyncio.sleep(2**retry_count)
        raise Exception(f"处理数据失败, 达到重试上限, data: {data}")

    async def _process_file(self, hdfs_file_path, process_func):
        """处理单个 gz 文件"""
        start_time = time.perf_counter()
        total_lines = self._count_total_lines(hdfs_file_path)
        processed_lines = 0
        pool = await self._get_pool()
        results = []

        for lines in self._batch_read_gz(hdfs_file_path):
            processing_start_time = time.perf_counter()  # 记录本批处理开始时间

            tasks = [self._process_data(line, process_func, pool) for line in lines]
            results.extend(await AsyncTaskPool(self.concurrency).run(tasks))

            processed_lines += len(lines)

            self._print_progress(hdfs_file_path, processed_lines, total_lines, start_time, processing_start_time)

        if self.result_dir is not None:
            self.client.write(
                self.result_dir.rstrip("/") + f"/{Path(hdfs_file_path).stem}",
                data=self._generate_write_data(results),
                overwrite=True,
                encoding=self.encoding,
            )

        # 最终进度显示
        self._print_final_progress(hdfs_file_path, processed_lines, total_lines, start_time)

    async def map(self, process_func: Callable[[str, Any], Awaitable[Any]]) -> None:
        gz_files = self._list_gz_files()
        await self._run_multi_process(gz_files, process_func)


# async def test_func(line: str, pool):
#     pass


# async def main():
#     await HDFSLineProcessor(
#         "/xx/xx",
#         db_path=Path(__file__).parent / "test.db",
#         concurrency=200,
#         batch_size=1000,
#         pool_factory=get_pool,
#         max_processes=2,
#         result_dir="/xx/xx_res",
#     ).map(test_func)


# if __name__ == "__main__":
#     asyncio.run(main())



# 储存单位
import os
import platform
import re

# 文件大小单位

SUFFIX = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB']
# 系统的分割符
__os_sep__ = "\\" if platform.system() == 'Windows' else os.sep
# 标准文件名正着
_filename_ascii_strip_re = re.compile(r'[^A-Za-z0-9_.-]')
# windows的设备文件
_windows_device_files = ('CON', 'AUX', 'COM1', 'COM2', 'COM3', 'COM4', 'LPT1',
                         'LPT2', 'LPT3', 'PRN', 'NUL')






class BaseList(object):

    def remove_null(self, lists):
        """
        清除列表的所有空字符串
        :param lists:
        :return:
        """
        return [r for r in lists if r != ""]

    def remove_blank_space(self, lists):
        """
        清除列表的所有空格数据,有其他字符的不算
        ["a","     ","  b"],　比如清除中间的　不清除ｂ
        :return:
        """
        return [r for r in lists if not r.isspace()]

    def clean_space_first_end(self, lists):
        """
        清理列表里每个字符串前后的空格
        :param lists:
        :return:
        """
        return [r.strip() for r in lists]

    def get_index(self, lists, obj):
        """
        从列表中找出某个值第一个匹配项的索引位置
        :return:
        """
        return lists.index(obj)



import time

from pandas._libs.tslibs.timestamps import Timestamp


def test_dataform_to_numpy():
    import pandas as pd
    import numpy as np
    dicts = {'A': 1.,
             'B': pd.Timestamp('20130102'),
             'C': pd.Series(1, index=list(range(4)), dtype='float32'),
             'D': np.array([3] * 4, dtype='int32'),
             'E': pd.Categorical(["test", "train", "test", "train"]),
             'F': 'foo'}
    from re_common.baselibrary.utils.basepandas import BasePandas
    bp = BasePandas()
    startTime = time.time()
    df = bp.dicts_to_dataform(dicts)
    # 数据类型一致时速度会很快
    print(bp.dataform_to_numpy(df))
    endTime = time.time()
    print(endTime - startTime)
    dicts = {'A': [1.0, Timestamp('2013-01-02 00:00:00'), 1.0, 3, 'test', 'foo'],
             'B': [Timestamp('2013-01-02 00:00:00'), Timestamp('2013-01-02 00:00:00'), 1.0, 3, 'test', 'foo'],
             'C': [1.0, Timestamp('2013-01-02 00:00:00'), 1.0, 3, 'test', 'foo'],
             'D': [3, Timestamp('2013-01-02 00:00:00'), 1.0, 3, 'test', 'foo'],
             'E': ['test', Timestamp('2013-01-02 00:00:00'), 1.0, 3, 'test', 'foo'],
             'F': ['foo', Timestamp('2013-01-02 00:00:00'), 1.0, 3, 'test', 'foo']}


    startTime = time.time()
    df = bp.dicts_to_dataform(dicts)
    # 数据类型一致时速度会很快
    print(bp.dataform_to_numpy(df))
    endTime = time.time()
    print(endTime - startTime)

test_dataform_to_numpy()



# -*- coding: utf-8 -*-
# @Time    : 2020/6/23 14:38
# @Author  : suhong
# @File    : wp_proxy_all.py
# @Software: PyCharm

"""
云代理 开放代理 刷新验证进入redis (db3 wp_proxy_all)
"""
import json
import random
import time

from re_common.baselibrary.utils.core.requests_core import USER_AGENTS

from re_common.facade.mysqlfacade import MysqlUtiles

from re_common.baselibrary.mthread.mythreading import ThreadInfo, ThreadVal, ThreadPoolManger

from re_common.baselibrary.mthread.MThreadingRun import MThreadingRun

from re_common.baselibrary.utils.baserequest import BaseRequest

from re_common.baselibrary.utils.myredisclient import MyRedis


class ProxyAll(object):
    def __init__(self, config="./db.ini"):
        self.config = config
        self.myredis = MyRedis(configpath=self.config, sesc='wp_proxy',is_conn_or_pipe=False)
        # self.myredis.get_pipeline()
        self.myredis.builder()

        self.Headers = {
            'Accept': '*/*',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko',
        }
        self.UserAgent = 'Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko'
        self.baserequest = BaseRequest()
        self.ProxyPoolTotal = set()
        self.starttime = time.time()
        self.mysqlutils_proxy = MysqlUtiles('./db.ini', "db_proxy")

    def get_redis_all(self):
        return self.myredis.getDataFromRedis()

    def get_proxy_ip3366(self, num=600):
        proxyPool = set()
        # url = "http://gea.ip3366.net/api/?key=20200622135212736&getnum={}&order=1&formats=2&proxytype=01".format(num)
        url = "http://gea.ip3366.net/api/?key=20200622135212736&getnum={}&formats=2&proxytype=01".format(num)
        try:
            bools, estring, r = self.baserequest.base_request(url,
                                                              headers=self.Headers,
                                                              marks=['Ip'])
            if bools:
                json_data = json.loads(r.text)
                for info in json_data:
                    proxy = info['Ip'] + ":" + str(info['Port'])
                    proxyPool.add(proxy)

                return True, proxyPool

            return False, proxyPool
        except Exception as e:
            print(e)
            return False, proxyPool


class YanzhengThreadRun(MThreadingRun):
    def __init__(self, num):
        super(YanzhengThreadRun, self).__init__(num)
        self.pro = ProxyAll()
        self.yz_right_set = set()
        self.is_clean_redis = False


    def getTask(self, *args, **kwargs):
        return []

    def setTask(self, results=None, *args, **kwargs):

        if self.thread_pool.work_queue.is_empty():

            redis_proxypool = self.pro.get_redis_all()
            self.is_clean_redis = True
            if time.time() - self.pro.starttime <= 5:
                time.sleep(6 - (time.time() - self.pro.starttime))
            bools, proxypool = self.pro.get_proxy_ip3366()
            result_pro = proxypool.union(redis_proxypool)
            if bools:
                for raw in result_pro:
                    self.add_job(self.func,raw)
            self.pro.starttime = time.time()


    def dealresult(self, *args, **kwargs):
        if self.is_clean_redis:
            # 清理reids
            self.pipe.delete(self.pro.myredis.RedisKey)
            self.is_clean_redis = False

        # 处理self.yz_right_set集合
        print('Write DataBase %s ...' % self.pro.myredis.RedisKey)
        self.pipe.sadd(self.pro.myredis.RedisKey, *self.results)
        curTime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
        self.pipe.hset('update_time', self.pro.myredis.RedisKey, curTime)
        self.pipe.execute()
        # 插入mysql
        for raw in self.results:
            self.deal_sql(raw)

    @ThreadPoolManger.thread_lock
    def deal_sql(self,raw):
        # 插入mysql统计每天获取的代理个数
        sql = "insert into gongwangproxy(proxy,cishu) Values('{}',1) on DUPLICATE key update cishu=cishu+1".format(raw)
        self.pro.mysqlutils_proxy.ExeSqlToDB(sql)



    def setProxy(self, proxysList=None):
        pass

    def is_break(self):
        return False

    def thread_pool_hook(self, threadinfo: ThreadInfo):
        # 设置代理线程不重启，默认会重启
        # if threadinfo.get_thread_name() == self.etn.proxythreadname:
        #     threadinfo.set_is_restart(True)
        # if threadinfo.get_thread_name() == self.etn.taskthreadname:
        #     threadinfo.set_is_restart(False)
        return {}

    def fun(self, threadval: ThreadVal, *args, **kwargs):
        """
        验证代理有效性,百度
        """
        raw = args[0]
        result_queue = threadval.get_result_queue()
        ppp = {
            'http': raw,
            'https': raw
        }
        try:
            url = "https://www.baidu.com/"
            bools, e, r = self.pro.baserequest.base_request(url,
                                                            headers=self.pro.Headers,
                                                            proxies=ppp,
                                                            timeout=5,
                                                            marks=['百度一下，你就知道'])
            if bools:
                result_queue.put(raw)
        except Exception as e:
            print(e)

        # 验证超星期刊
        # yzurl = "http://qikan.chaoxing.com/mag/infos?mags=ea15bb11cfca2424ae72402ca8461604"
        # mags = "ea15bb11cfca2424ae72402ca8461604"
        # HEADER = {
        #     'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        #     'User-Agent': random.choice(USER_AGENTS),
        #     'Accept-Encoding': 'gzip, deflate',
        #     'Referer': 'http://qikan.chaoxing.com',
        # }
        # HEADER['Referer'] = 'http://qikan.chaoxing.com/mag/infos?mags=' + mags
        # try:
        #     BoolResult, errString, r = self.pro.baserequest.base_request(url=yzurl,
        #                                                            headers=HEADER,
        #                                                            timeout=(5, 10),
        #                                                            proxies=ppp,
        #                                                            marks=['Fbookright fl'])
        #     if BoolResult:
        #         result_queue.put(raw)
        # except Exception as e:
        #     print(e)


if __name__ == '__main__':
    # p = ProxyAll()
    # p.get_redis_all()
    yz = YanzhengThreadRun(40)
    yz.run()


import json

from re_common.baselibrary.utils.core.requests_core import MsgCode


class MRequest(object):
    POST = "post"
    GET = "get"

    def __init__(self, logger=None):
        if logger is None:
            from re_common.baselibrary import MLogger
            logger = MLogger().streamlogger
        self.logger = logger
        self.html = None
        self.html_bytes = None
        self.resp = None
        self.marks = []
        self.middler_list = [self.is_null_html_middlerwares, self.status_code_middlerwares, self.end_middlerwares,
                             self.marks_middlerwares]
        self.status_code = None
        self.header = None
        self.refer = None
        self.proxy = None
        self.url = None
        self.params = None
        self.data = None
        self.cookies = None
        self.sn = None
        self.files = None
        self.auth = None
        self.timeout = None
        self.allow_redirects = True
        self.hooks = None
        self.stream = None
        self.verify = None
        self.cert = None
        self.json = None
        self.resp_encoding = None
        self.resp_errors = "strict"
        self.resp_dicts = {"code": self.status_code, "msg": ""}
        self.allow_resp_text = True
        self.allow_resp_bytes = False
        self.middler_para = {}
        self.skip_auto_headers = None
        self.force_close = None  # "true" or "false"

    def set_force_close(self, true_or_false):
        self.force_close = true_or_false

    def set_skip_auto_headers(self, lists):
        """
        设置skip_auto_headers的参数
        skip_auto_headers 用法: 不对列表内的对应参数进行自动生成
        :return:
        """
        self.skip_auto_headers = lists
        return self

    def set_middler_para(self, dicts):
        """
        设置middler的参数
          def status_code_middlerwares(self, status_code=200):
          可以设置
          dicts = {”status_code_middlerwares“:{"status_code":200}}
        :return:
        """
        self.middler_para = dicts

    def set_allow_redirects(self, flag=True):
        """
        设置是否允许重定向
        :param flag:
        :return:
        """
        self.allow_redirects = flag
        return self

    def set_allow_resp_text(self, flag=True):
        """
        设置是否返回response内容
        解决重定向设置为false时内容无法解析导致编码错误
        :param flag:
        :return:
        """
        self.allow_resp_text = flag
        return self

    def set_allow_resp_bytes(self, flag=True):
        """
        设置是否返回response内容的字节信息，grpc可以用到
        :param flag:
        :return:
        """
        self.allow_resp_bytes = flag
        return self

    def set_resp_errors(self, errors):
        """
        设置结果是否忽略乱码错误 可以设置为 ignore
        errors 为 decode 的属性 errors=errors
        :param errors:
        :return:
        """
        self.resp_errors = errors
        return self

    def set_resp_encoding(self, encode):
        """
        设置结果编码
        :param encode:
        :return:
        """
        self.resp_encoding = encode
        return self

    def set_html(self, html):
        """
        设置html
        :param html:
        :return:
        """
        self.html = html
        return self

    def set_status_code(self, status_code):
        """
        设置状态码
        :param status_code:
        :return:
        """
        self.status_code = status_code
        self.resp_dicts["code"] = self.status_code
        return self

    def set_middler_list(self, lists):
        """
        设置需要验证的步骤
        :param lists:
        :return:
        """
        if lists is not None:
            self.middler_list = lists
        return self

    def set_marks(self, marks: list):
        """
        设置验证码
        :param marks:
        :return:
        """
        self.marks = marks
        return self

    def set_timeout(self, timeout):
        """
        设置超时
        :param timeout:
        :return:
        """
        self.timeout = timeout
        return self

    def set_header(self, header):
        """
        设置header
        :return:
        """
        self.header = header
        return self

    def set_refer(self, refer):
        """
        设置header中的refer，每次请求有可能变化
        :return:
        """
        self.refer = refer
        return self

    def set_proxy(self, proxy):
        """
        设置代理
        :return:
        """
        self.proxy = proxy
        return self

    def set_url(self, url):
        """
        设置请求的url
        :return:
        """
        self.url = url
        return self

    def set_params(self, params):
        """
        get 请求参数
        :return:
        """
        self.params = params
        return self

    def set_data(self, data):
        """
        设置请求参数
        :return:
        """
        self.data = data
        return self

    def set_cookies(self, cookies):
        """
        设置cookie
        :return:
        """
        self.cookies = cookies
        return self

    def set_sn(self, sn):
        """
        设置会话
        :return:
        """
        self.sn = sn
        return self

    def close_sn(self):
        """
        关闭会话
        :return:
        """
        if self.sn is not None:
            self.sn.close()

    def builder(self):
        """
        建造成需要的对象用于接下来请求使用
        :return:
        """
        if self.refer != "":
            self.header["refer"] = self.refer

    def get(self):
        """
        get 请求
        :return:
        """
        pass

    def post(self):
        """
        post 请求
        :return:
        """

    def on_request_start(self):
        """
        请求前的钩子函数
        :return:
        """

    def on_request_end(self):
        """
        请求结束的钩子函数
        :return:
        """

    def status_code_middlerwares(self, status_code=200):
        """
        验证返回码
        :return:
        """
        if "status_code_middlerwares" in self.middler_para.keys():
            if "status_code" in self.middler_para["status_code_middlerwares"].keys():
                status_code = self.middler_para["status_code_middlerwares"]["status_code"]
        if self.status_code != status_code:
            self.resp_dicts["code"] = self.status_code
            self.resp_dicts["msg"] = "status_code err {},{}".format(self.status_code,self.url)
            return False, self.resp_dicts
        return True, self.resp_dicts

    def end_middlerwares(self, endstring="</html>"):
        """
        必须以什么结束
        :return:
        """
        if "end_middlerwares" in self.middler_para.keys():
            if "endstring" in self.middler_para["end_middlerwares"].keys():
                endstring = self.middler_para["end_middlerwares"]["endstring"]
        if not self.html.endswith(endstring):
            self.resp_dicts["code"] = MsgCode.END_STRING_ERROR
            self.resp_dicts["msg"] = "not endswith {},{}".format(endstring,self.url)
            return False, self.resp_dicts
        return True, self.resp_dicts

    def is_none_html_middlerwares(self):
        """
        空的html 在使用某些代理时 如果https 写成了http,会有这种情况
        建议所有http的网站使用这个验证器以监控网站何时变成http
        :return:
        """
        if self.html is not None:
            if len(self.html) == 0:
                self.resp_dicts["code"] = MsgCode.NONE_HTML
                self.resp_dicts["msg"] = "空的html"
                return False, self.resp_dicts
        else:
            self.resp_dicts["code"] = MsgCode.NONE_HTML
            self.resp_dicts["msg"] = "空的html"
            return False, self.resp_dicts
        return True, self.resp_dicts

    def is_null_html_middlerwares(self):
        """
        api 如果忘记return 会返回 null 字符串
        :return:
        """
        if self.html == "null":
            self.resp_dicts["code"] = MsgCode.NONE_HTML
            self.resp_dicts["msg"] = "api没有return导致返回 null"
            return False, self.resp_dicts
        return True, self.resp_dicts

    def have_end_middlerwares(self, havestring="</html>"):
        if "have_end_middlerwares" in self.middler_para.keys():
            if "havestring" in self.middler_para["have_end_middlerwares"].keys():
                havestring = self.middler_para["have_end_middlerwares"]["havestring"]

        if self.html.find(havestring) == -1:
            self.resp_dicts["code"] = MsgCode.END_STRING_ERROR
            self.resp_dicts["msg"] = "not have endswith {}".format(havestring)
            return False, self.resp_dicts
        return True, self.resp_dicts

    def marks_middlerwares(self):
        """
        建议至少两个，
        一个是id 防止因为cookie请求到其他的页面
        一个为验证该html关键词，用于改版预测
        :param marks: 一个列表
        :return:
        """
        mark_str = ""
        for mark in self.marks:
            if self.html.find(mark) == -1:
                mark_str = mark_str + mark + ";"

        if mark_str != "":
            self.resp_dicts["code"] = MsgCode.MARK_ERROR
            self.resp_dicts["msg"] = "mark Feature err: {}".format(mark_str)
            return False, self.resp_dicts
        else:
            return True, self.resp_dicts

    def is_json_middlerwares(self):
        """
        验证返回结果是否是json
        :return:
        """
        try:
            dic = json.loads(self.html)
        except Exception as e:
            self.resp_dicts["code"] = MsgCode.NOT_IS_JSON
            self.resp_dicts["msg"] = "not is json error, {}".format(repr(e))
            return False, self.resp_dicts
        return True, self.resp_dicts

    def fail_middlerwares(self):
        """
        当请求失败时,返回错误信息
        废弃这个方法 请使用msg_status_code_middlerwares
        :return:
        """
        try:
            dic = json.loads(self.html)
            if dic["status"] == "SUCCESS":
                return True, self.resp_dicts
            else:
                self.resp_dicts["code"] = MsgCode.STATUS_ERROR
                self.resp_dicts["msg"] = "Status error"
                return False, self.resp_dicts
        except Exception as e:
            self.resp_dicts["code"] = MsgCode.NOT_IS_JSON
            self.resp_dicts["msg"] = "not is json error, {}".format(repr(e))
            return False, self.resp_dicts

    def msg_status_code_middlerwares(self, status=["SUCCESS"], msg_code=[200]):
        """
        status 和 msg_code 错误拦截
        :return:
        """
        if "msg_status_code_middlerwares" in self.middler_para.keys():
            if "status" in self.middler_para["msg_status_code_middlerwares"].keys():
                status = self.middler_para["msg_status_code_middlerwares"]["status"]
            if "msg_code" in self.middler_para["msg_status_code_middlerwares"].keys():
                msg_code = self.middler_para["msg_status_code_middlerwares"]["msg_code"]
        try:
            dic = json.loads(self.html)
            if dic["status"] not in status:
                self.resp_dicts["code"] = MsgCode.STATUS_ERROR
                self.resp_dicts["msg"] = "error, {}".format(self.html)
                return False, self.resp_dicts
            if dic["msg_code"] not in msg_code:
                self.resp_dicts["code"] = MsgCode.CODE_ERROR
                self.resp_dicts["msg"] = "error data code, {}".format(dic["msg_code"])
                return False, self.resp_dicts
        except Exception as e:
            self.resp_dicts["code"] = MsgCode.NOT_IS_JSON
            self.resp_dicts["msg"] = "not is json error, {}".format(repr(e))
            return False, self.resp_dicts
        return True, self.resp_dicts



import sqlite3
import requests
import pandas as pd
import os
import json
import traceback


# c1d3a814-1a02-4bbd-b5c2-f756fef92cb8: b层机器人消息群-非聊天 的 pythonspark
# 013547da-3d78-4a7f-b4a7-e668b192c293: b层机器人消息群-非聊天 的 数仓B层服务端部署通知

# 发送消息到企业微信机器人
# vx_key: string类型，自己的企业微信机器人的key
# s:string类型，要发送的消息
def send_vx(vx_key, s, i=0):
    vx_url = 'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=' + vx_key
    headers = {"Content-Type": "text/plain"}
    data = {
        "msgtype": "text",
        "text": {
            "content": s,
        }
    }
    if i > 3:
        raise Exception(str(traceback.format_exc()))
    try:
        requests.post(url=vx_url, headers=headers, json=data, timeout=30)
    except:
        i = i + 1
        send_vx(vx_key, str(traceback.format_exc()), i)


# 发送文件到企业微信机器人
# vx_key: string类型，自己的企业微信机器人的key
# file_path: string类型，文件地址
def post_file(vx_key, file_path):
    id_url = 'https://qyapi.weixin.qq.com/cgi-bin/webhook/upload_media?key=' + vx_key + '&type=file'
    wx_url = 'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=' + vx_key
    data = {'file': open(file_path, 'rb')}
    response = requests.post(url=id_url, files=data)
    json_res = response.json()
    media_id = json_res['media_id']
    data = {"msgtype": "file",
            "file": {"media_id": media_id}
            }
    try:
        requests.post(url=wx_url, json=data)
    except:
        send_vx(send_vx, str(traceback.format_exc()))


# data: dataframe|string|dict|list|tuple|array
# file_name: 带后缀的完整文件名
# file_type: 文件类型，包括csv、excel、txt、json、sql
def file_to_vx(vx_key, data, file_name):
    file_type = file_name.split('.')[-1]
    if file_type == "xls" or file_type == "xlsx":
        file_type = "excel"
    current_dir = os.getcwd()
    temp_dir = os.path.join(os.getcwd(), "tmp")
    if os.path.exists(temp_dir):
        pass
    else:
        os.makedirs(temp_dir)
    file_path = current_dir + "/" + file_name
    try:
        if isinstance(data, pd.DataFrame) and file_type != "txt":
            if file_type == "db3":
                conn = sqlite3.connect(file_path)
                data.to_sql('base_table', conn, if_exists='replace', index=False)
                post_file(vx_key, file_path)
            else:
                code_str = "data.to_" + file_type + "(file_path,index=False)"
                eval(code_str)
                post_file(vx_key, file_path)
            os.system('rm ' + file_path + '')
        else:
            if isinstance(data, dict):
                data_str = json.dumps(data, ensure_ascii=False)
            elif isinstance(data, list):
                data_str = ""
                for i in data:
                    if isinstance(i, dict):
                        data_str = data_str + json.dumps(i, ensure_ascii=False) + "\n"
                    else:
                        data_str = data_str + str(i) + "\n"
            else:
                data_str = str(data)
            print(data_str[:100])
            with open(file_path, 'w', encoding='utf-8') as f:
                f.write(data_str)
            post_file(vx_key, file_path)
    except:
        send_vx(vx_key, str(traceback.format_exc()))
    os.system('rm -r' + temp_dir + '')



import streamlit as st
import pandas as pd
import numpy as np

st.title('Uber pickups in NYC')

DATE_COLUMN = 'date/time'
DATA_URL = ('https://s3-us-west-2.amazonaws.com/streamlit-demo-data/uber-raw-data-sep14.csv.gz')

@st.cache
def load_data(nrows):
    data = pd.read_csv(DATA_URL, nrows=nrows)
    lowercase = lambda x: str(x).lower()
    data.rename(lowercase, axis='columns', inplace=True)
    data[DATE_COLUMN] = pd.to_datetime(data[DATE_COLUMN])
    return data

# Create a text element and let the reader know the data is loading.
data_load_state.text("Done! (using st.cache)")

# Load 10,000 rows of data into the dataframe.
data = load_data(10000)
# Notify the reader that the data was successfully loaded.
data_load_state.text('Loading data...done!')


import re
import threading
from html.parser import HTMLParser
from itertools import combinations

import regex
import unicodedata

from re_common.v2.baselibrary.utils.string_smi import JaroDamerauLevenshteinMaxSim


def bj2qj(src):
    if src is None:
        return src

    DBC_SPACE = ' '
    SBC_SPACE = '　'
    DBC_CHAR_START = 33
    DBC_CHAR_END = 126
    CONVERT_STEP = 65248

    buf = []
    for char in src:
        if char == DBC_SPACE:
            buf.append(SBC_SPACE)
        elif DBC_CHAR_START <= ord(char) <= DBC_CHAR_END:
            buf.append(chr(ord(char) + CONVERT_STEP))
        else:
            buf.append(char)

    return ''.join(buf)


def qj2bj(text):
    if text is None:
        return text
    # 预构建全角到半角的转换映射表（只需构建一次）
    if not hasattr(qj2bj, 'trans_table'):
        trans_map = {}
        # 处理全角空格
        trans_map[0x3000] = 0x0020
        # 处理全角字符范围FF01-FF5E
        for code in range(0xFF01, 0xFF5F):
            trans_map[code] = code - 0xFEE0
        # 创建转换表（字符到字符的映射）
        qj2bj.trans_table = str.maketrans(
            {chr(k): chr(v) for k, v in trans_map.items()}
        )
    # 使用预编译的转换表进行高效替换
    return text.translate(qj2bj.trans_table)


"""
总结对比表
规范名	处理步骤	组合方式	兼容性归一化	主要用途
NFC	规范分解 → 规范组合	组合	否	保留预组合字符，文本呈现和存储
NFD	规范分解	不组合	否	拆解字符，便于逐字符处理
NFKC	兼容性分解 → 规范组合	组合	是	消除兼容差异，文本比较和索引
NFKD	兼容性分解 → 规范分解	不组合	是	最大程度拆解，文本分析和预处理
"""


def get_diacritic_variant(char1):
    """
    NFD: 规范分解（Normalization Form D）
    把字符拆分为基本字符 + 变音符号

    但不处理兼容字符（如连字）

    print(unicodedata.normalize('NFD', 'é'))  # 输出: 'é'（e + 组合符号） # 这里看起来是1个字符 len 其实是2
    print(unicodedata.normalize('NFD', 'ﬂ'))  # 输出: 'ﬂ'（不变化）

    """
    # 将字符转换为标准的 Unicode 形式
    normalized_char1 = unicodedata.normalize('NFD', char1)

    # 获取基本字符（去掉变音符号）
    base_char1 = ''.join(c for c in normalized_char1 if unicodedata.category(c) != 'Mn')

    # 判断基本字符是否相同
    return base_char1


def normalize_nfkc(strs: str) -> str:
    """
    NFKC: 兼容字符归一化 + 组合（Normalization Form Compatibility Composition）
    把 连字、圈数字、全角字符 等兼容字符转换为标准形式

    同时做字符合并（例如 é 不再是 e+´，而是一个字符）
    print(unicodedata.normalize('NFKC', 'ﬂ'))   # 输出: 'fl'
    print(unicodedata.normalize('NFKC', '①'))   # 输出: '1'
    print(unicodedata.normalize('NFKC', 'Ａ'))  # 输出: 'A'
    """
    return unicodedata.normalize('NFKC', strs.strip())


def get_alphabetic_ratio(text: str) -> float:
    # 返回字母型字符所占比例
    if not text:
        return 0

    text = re.sub(r'\d+', '', text)

    # 正则表达式匹配字母型文字（包括拉丁字母、希腊字母、西里尔字母、阿拉伯字母等）
    alphabetic_pattern = (
        r"[\u0041-\u005A\u0061-\u007A"  # 拉丁字母 (A-Z, a-z)
        r"\u00C0-\u00FF"  # 带重音符号的拉丁字母 (À-ÿ)
        r"\u0080–\u00FF"  # 拉丁字母补充1
        r"\u0100–\u017F"  # 拉丁字母扩展A
        r"\u1E00-\u1EFF"  # 拉丁扩展 (Latin Extended Additional)
        r"\u0180-\u024F"  # 拉丁扩展-B (Latin Extended-B)
        r"\u2C60-\u2C7F"  # 拉丁扩展-C (Latin Extended Additional)
        r"\uA720-\uA7FF"  # 拉丁扩展-D (Latin Extended Additional)
        r"\uAB30-\uAB6F"  # 拉丁扩展-E (Latin Extended Additional)
        r"]"
    )

    # 使用正则表达式过滤出语言文字
    clean_text = regex.sub(r"[^\p{L}]", "", text)

    if len(clean_text) == 0:
        return 1.0

    # 匹配所有字母型字符
    alphabetic_chars = re.findall(alphabetic_pattern, clean_text)

    # 返回字母型字符所占比例
    return len(alphabetic_chars) / len(clean_text)


class HTMLTextExtractor(HTMLParser):
    _thread_local = threading.local()  # 线程局部存储

    def __init__(self):
        super().__init__()
        self.reset_state()

    def handle_starttag(self, tag, attrs):
        if tag in ('script', 'style'):
            self.skip = True

    def handle_endtag(self, tag):
        if tag in ('script', 'style'):
            self.skip = False

    def handle_data(self, data):
        if not self.skip and data.strip():
            self.text.append(data)

    def reset_state(self):
        self.reset()
        self.text = []
        self.skip = False

    def get_text(self):
        return ''.join(self.text).strip()

    @classmethod
    def get_parser(cls):
        # 每个线程获取独立实例
        if not hasattr(cls._thread_local, 'parser'):
            cls._thread_local.parser = cls()
        return cls._thread_local.parser


# def clean_html(html):
#     parser = HTMLTextExtractor.get_parser()
#     parser.reset_state()
#     parser.feed(html)
#     parser.close()
#     return parser.get_text()

# def clean_html(html):
#     """使用 Parsel 提取 HTML 中的纯文本"""
#     sel = Selector(text=html, type='html')
#     # 提取所有文本（包括子元素的文本）
#     text = sel.xpath("string()").getall()
#     return "".join(text).strip()


def clean_html(html):
    if "<" in html:
        from bs4 import BeautifulSoup
        try:
            soup = BeautifulSoup(html, "lxml")
            return soup.get_text()
        except:
            soup = BeautifulSoup(html, "html5lib")
            return soup.get_text()
    return html


def remove_spaces_between_chinese_characters(text):
    """
    匹配中文间的空格并替换为空字符串

    这里没有选取 后面的一些扩展分区 是那些分区比较分散 都写进来消耗性能,
    认为只包含这些也够用了
    """
    pattern = r'(?<=[\u3400-\u9fff])\s+(?=[\u3400-\u9fff])'
    return re.sub(pattern, '', text)


sim_utils = JaroDamerauLevenshteinMaxSim()


def group_similar_texts(texts, threshold=0.9):
    """根据相似度对文本进行分组"""
    from re_common.v2.baselibrary.utils.string_clear import rel_clear
    n = len(texts)
    # 创建邻接表表示图
    graph = [[] for _ in range(n)]
    # 计算所有文本对的相似度并构建图
    for i, j in combinations(range(n), 2):
        similarity = sim_utils.get_sim(rel_clear(texts[i]), rel_clear(texts[j]))
        if similarity >= threshold:
            graph[i].append(j)
            graph[j].append(i)

    visited = [False] * n
    groups = []

    # 使用DFS找到连通分量
    def dfs(node, group):
        visited[node] = True
        group.append(node)
        for neighbor in graph[node]:
            if not visited[neighbor]:
                dfs(neighbor, group)

    # 找到所有连通分量
    for i in range(n):
        if not visited[i]:
            current_group = []
            dfs(i, current_group)
            groups.append(current_group)

    return groups


def get_group_abstract(lists):
    """
    这是一个 分组程序 ，会根据简单的连通图分组
    lists: [(id,txt),...]
    return: all_list 返回一个二维列表 每个列表里面是id 每个列表为一个分组
    """
    abstract_list = [i[1] for i in lists]
    keyid_list = [i[0] for i in lists]
    groups = group_similar_texts(abstract_list, threshold=0.9)
    all_list = []
    for group in groups:
        t_list = []
        for text_idx in group:
            t_list.append(keyid_list[text_idx])
        all_list.append(t_list)
    return all_list


def clean_unicode_alnum(text: str) -> str:
    """
    清除所有非 Unicode 字母或数字的字符。

    参数:
        text (str): 输入文本。

    返回:
        str: 只包含 Unicode 字母和数字的文本。
    \p{N} 匹配所有 Unicode 数字字符 包括非阿拉伯数字字符
    \p{L} 匹配所有语言字符
    """
    return regex.sub(r"[^\p{L}\p{N}]+", "", text)



def assign_group_id(rows: list, sub_db_order: list):
    subdb_keyid_map = {row.sub_db_id: row.keyid for row in rows}
    for sub_db_id in sub_db_order:
        if keyid := subdb_keyid_map.get(sub_db_id):
            return keyid, len(rows), rows
    return rows[0].keyid, len(rows), rows



def get_attrs(settings):
    """
    获取模块参数转换为字典
    :param settings:
    :return:
    """
    attr = {}
    for i in (dir(settings)):
        if i.isupper():
            attr[i] = getattr(settings,i)
    return attr


from re_common.baselibrary.tools.move_mongo.mongo_table_to_file import Configs, MongoToFile

conf = Configs()


def hook_doc(doc):
    doc_info = doc["step_info"]
    id_ = doc_info["id"]
    url = f"https://wiki.mbalib.com/wiki/{id_}"
    doc_info["url"] = url
    return doc


mtf = MongoToFile(conf)
mtf.hook_doc = hook_doc
mtf.init_conn_mongodb()
mtf.open_file()
mtf.asyncio_run()



from re_common.baselibrary.utils.basehttpx import BaseHttpx
from re_common.baselibrary.utils.baserequest import BaseRequest
import re

url = "https://www.ip.cn/"


def basehttpx_test_ip(proxy):
    patten = "<code>(.*?)</code>"
    bshttpx = BaseHttpx()
    BoolResult, errString, r = bshttpx.base_sn_httpx(url=url,
                                                     proxies=proxy,
                                                     headers="")
    ip = re.findall(patten,r.text)
    if len(ip) == 0:
        print(r.text)
    else:
        print(ip)

def baserequest_test_ip(proxy):
    patten = "Your IP</span>: (.*?)</span>"
    bsrequests = BaseRequest()
    bsrequests.is_use_proxy(True)
    bsrequests.set_proxy(proxy)
    BoolResult, errString, r = bsrequests.base_request(url=url)
    ip = re.findall(patten, r.text)
    if len(ip) == 0:
        print(r.text)
    else:
        print(ip)


if __name__ == '__main__':
    basehttpx_test_ip("192.168.31.176:8012")
    baserequest_test_ip("192.168.31.176:8012")


import Levenshtein
import jellyfish
from rapidfuzz.distance import DamerauLevenshtein, Hamming, Indel, LCSseq, OSA


class BaseStringSimilarity(object):

    @classmethod
    def levenshtein_similarity(cls, str1, str2) -> float:
        """
        返回 两个字字符串之间的编辑距离 分数
        """
        # 编辑距离长度
        distance = Levenshtein.distance(str1, str2)
        # 以最长字符串为除数算分
        similarity = 1 - (distance / max(len(str1), len(str2)))
        return similarity

    @classmethod
    def damerau_normalized_distance_similarity(cls, str1, str2) -> float:
        """
          # 计算 归一化的编辑距离，取值范围 [0, 1]，值越小表示越相似。 一般不以小评估分 所以不用
        similarity = DamerauLevenshtein.normalized_distance(str1, str2)
        作用：计算 相似度得分，取值范围 [0, max_len]，值越大表示越相似。
        print(DamerauLevenshtein.similarity(str1, str2))
        """
        # 该算法与 cls.levenshtein_similarity 算法一致 只是 编辑距离的得值不一样
        similarity = DamerauLevenshtein.normalized_similarity(str1, str2)
        return similarity

    @classmethod
    def indel_levenshtein_similarity(cls, str1, str2) -> float:
        """
        本质上使用的 是 Indel.normalized_similarity(str1,str2) 方法

        计算 str1 和 str2 之间的 Indel 距离（插入和删除操作的最小次数）
        Indel.distance(str1, str2)
        计算 标准化后的 Indel 距离，取值范围在 [0, 1] 之间，其中 0 表示完全相同，1 表示完全不同。 ``distance / (len1 + len2)``.
        Indel.normalized_distance(str1, str2)
        计算 [max, 0] 范围内的 Indel 相似度。计算公式为“(len1 + len2) - distance”
        Indel.similarity(str1, str2)
        计算 [0, 1] 范围内的归一化插入/缺失相似度。计算公式为“1 - normalized_distance”
        Indel.normalized_similarity(str1, str2)

        """
        # 计算相似度（0到1之间的值，1表示完全相同）
        similarity = Levenshtein.ratio(str1, str2)
        return similarity

    @classmethod
    def jaro_similarity(cls, str1, str2) -> float:
        """
        Jaro 相似度是一种用于测量两个字符串相似度的算法，主要考虑：
        匹配的字符
        字符顺序
        字符转置（位置交换）

        与 Jaro.normalized_similarity(str1,str2) 一致
        """
        return jellyfish.jaro_similarity(str1, str2)

    @classmethod
    def jaro_winkler_similarity(cls, str1, str2) -> float:
        """
        Jaro-Winkler 是 Jaro 的改进版，对前缀匹配给予更多权重

        与 JaroWinkler.normalized_similarity(str1,str2) 结果一致

        print(JaroWinkler.distance(str1, str2))
        与 print(JaroWinkler.normalized_distance(str1, str2)) 结果一致

        print(JaroWinkler.similarity(str1, str2))
        与 print(JaroWinkler.normalized_similarity(str1,str2)) 结果一致
        """
        return jellyfish.jaro_winkler_similarity(str1, str2)

    @classmethod
    def osa_similarity(cls, str1, str2) -> float:
        """
        计算 [0, 1] 范围内的归一化最佳字符串比对 (OSA) 相似度。

        计算公式为“1 - normalized_distance”
        """
        return OSA.normalized_similarity(str1, str2)

    @classmethod
    def lcs_seq_similarity(cls, str1, str2) -> float:
        """
        计算 [0, 1] 范围内的归一化 LCS 相似度。
        计算公式为“1 - normalized_distance”
        """
        return LCSseq.normalized_similarity(str1, str2)

    @classmethod
    def lcs_seq_distance(cls, str1, str2) -> int:
        """
        LCSseq.distance 是 RapidFuzz 库中的一个方法，用于计算两个字符串之间的 最长公共子序列（Longest Common Subsequence, LCS）距离。
        LCS 是指两个字符串中 按顺序出现但不一定连续 的最长子序列。例如：
        "abcde" 和 "ace" 的 LCS 是 "ace"（长度 3）。
        "Druitt, Robert" 和 "Druitt R." 的 LCS 可能是 "Druitt R"（长度 8）。
        计算 [0, max] 范围内的 LCS 距离。
        计算公式为“max(len1, len2) - 相似度”。
        """
        return LCSseq.distance(str1, str2)

    @classmethod
    def osa_distance(cls, str1, str2) -> int:
        """
        OSA.distance（Optimal String Alignment，最优字符串对齐距离）是 RapidFuzz 库中的一个方法，用于计算两个字符串之间的 编辑距离（Edit Distance），但比标准的 Levenshtein 距离 限制更严格。

        OSA 额外允许 相邻字符交换（Transposition），但限制比 Damerau-Levenshtein 更严格（Damerau 允许多次交换，而 OSA 仅限一次）。
        """
        return OSA.distance(str1, str2)

    @classmethod
    def levenshtein_distance(cls, str1, str2) -> int:
        """
        返回 两个字字符串之间的编辑距离 分数
        标准 Levenshtein 距离 允许 插入、删除、替换 三种操作，但不允许 相邻字符交换（transposition）

        jellyfish.levenshtein_distance(str1,str2) 该方法结果与 本方法一致

        print(Jaro.distance(str1, str2))
        与 print(Jaro.normalized_distance(str1, str2)) 结果一致

        print(Jaro.similarity(str1, str2))
        与 print(Jaro.normalized_similarity(str1,str2)) 结果一致
        """
        # 编辑距离长度
        distance = Levenshtein.distance(str1, str2)
        print(jellyfish.levenshtein_distance(str1, str2))
        return distance

    @classmethod
    def indel_distance(cls, str1, str2) -> int:
        """
        Indel（Insertion + Deletion）距离是 仅考虑插入和删除操作 的编辑距离，不考虑替换操作。
        """
        return Indel.distance(str1, str2)

    @classmethod
    def damerau_levenshtein_distance(cls, str1, str2) -> int:
        """
        Damerau-Levenshtein 距离是 Levenshtein 距离的修改，它将换位（例如将 ifsh 表示为 fish）计为一次编辑
        """
        # 编辑距离长度
        distance = jellyfish.damerau_levenshtein_distance(str1, str2)
        print(DamerauLevenshtein.distance(str1, str2))
        return distance

    @classmethod
    def hamming_distance(cls, str1, str2) -> int:
        return Hamming.distance(str1, str2)

# str1 = "primulina elegant ladyis a new culitvar developed by crossing seed parent primulina medica and pollen parent primulina longii it has fresh and elegant flowershigh ornamental value and strong shade tolerance it is easy to cultivate and propagate"
# str2 = "primulinaelegant labyis a new cultivar developed by crossing seed parent primulina medica and pollen parent primulina longii it has fresh and elegant flowershigh ornamental value and strong shade tolerance it is easy to cultivate and propagate 2019 editorial office of acta horticulturae sinica all rights reserved"
# # str1 = "primulina elegant ladyis a new cultivar developed by crossing seed parent primulina medica and pollen parent primulina longii it has fresh and elegant flowershigh ornamental value and strong shade tolerance it is easy to cultivate and propagate"
# # str2 = "primulinaelegant ladyis a new cultivar developed by crossing seed parent primulina medica and pollen parent primulina longii it has fresh and elegant flowershigh ornamental value and strong shade tolerance it is easy to cultivate and propagate 2019 editorial office of acta horticulturae sinica all rights reserved"



import ctypes
import functools
import inspect
import sys
import threading
# 创建一个有4个线程的线程池
import time
import traceback
from threading import Thread

from re_common.baselibrary.utils.basequeue import BaseQueue
from re_common.facade.loggerfacade import get_streamlogger


class ThreadInfo(object):
    """
    该类存储着线程的信息，原本使用一个空的字典进行存储，
    但字典的结构不可见且不清晰，故使用类进行结构化展示
    """

    def __init__(self):
        self.is_restart = True

    def set_thread(self, thread):
        self.thread = thread
        return self

    def get_thread(self):
        return self.thread

    def set_thread_name(self, thread_name):
        self.thread_name = thread_name
        return self

    def get_thread_name(self):
        return self.thread_name

    def set_kwargs(self, kwargs):
        self.kwargs = kwargs
        return self

    def get_kwargs(self):
        return self.kwargs

    def set_args(self, args):
        self.args = args
        return self

    def get_args(self):
        return self.args

    def set_dicts(self, dicts):
        self.dicts = dicts
        return self

    def get_dicts(self):
        return self.dicts

    def set_is_restart(self, is_restart: bool):
        """
        设置该线程是否应该重启
        :param is_restart:
        :return:
        """
        self.is_restart = is_restart
        return self

    def get_is_restart(self):
        return self.is_restart


class ThreadPoolManger(object):
    """
    线程池管理器
    """
    # 线程锁
    lock = threading.Lock()

    def __init__(self, thread_num, logger=None):
        if logger:
            self.logger = logger
        else:
            self.logger = get_streamlogger()
        self.prefix = "my_threading_task_"
        # 任务队列
        self.work_queue = BaseQueue(100)
        # 结果队列 有时候结果需要在主线程处理或单独线程处理时使用
        self.result_queue = BaseQueue(100)
        # 线程队列 主要管线程间的通信
        self.thread_queue = BaseQueue(10)
        # 线程池 里面保存着线程对象
        # self.threadpool = list()
        # # 线程名列表
        # self.initThreadsName = set()
        # 应该赋予一个方法 用于回调
        self.callback = None
        # 线程池的字典 有时候需要在线程池中维护一些数据，所以没有采用原来的只维护线程对象
        self.thread_pool_dicts = {}

        # 线程数量 反馈当前工作线程数量
        self.thread_num = thread_num
        # 工作线程定量 该变量是为了维持线程的数量一定
        self.max_workers = thread_num
        # 是否是动态最大值
        self.is_static_max = True
        # 线程的参数　定义线程时使用
        self.args = ()
        self.kwargs = {}

        # # 各种非工作线程池
        # self.especial_thread_pool = list()
        # # 特殊线程名
        # self.especial_threads_Name = set()

        # 各种非工作线程池 特殊线程名
        self.especial_thread_pool_dicts = {}

        # 该标志将用于sql间的同步  现在发现一个问题 在 线程中执行select会出现
        # Command Out of Sync 错误  所以启用这个信号
        # 这个错误表示 对于同一个连接 在不同线程中当一个线程发出sql且没有返回结果期间，
        # 另一个线程页发出了请求，这时mysql的第一个请求在结果缓存中还没有返回 会冲突报错
        # 这里主要是设置任务和处理结果用到同一个连接导致
        self.event = threading.Event()
        self.event_set()
        # 独立的线程变量，很多时候 多线程变量是共享且线程不安全的，
        # 该变量保证了线程内变量的安全与threadVal的概念相同
        self.localVal = threading.local()

        # # 初始化线程到线程池
        self.__init_threading_pool(self.thread_num)

    @classmethod
    def thread_lock(cls, func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                cls.lock.acquire()
                return func(*args, **kwargs)
            finally:
                cls.lock.release()

        return wrapper

    def set_args(self, args):
        """
        设置元组参数
        :param args:
        :return:
        """
        self.args = args

    def set_kwargs(self, kwargs):
        """
        设置字典参数
        :param kwargs:
        :return:
        """
        self.kwargs = kwargs

    def set_is_static_max(self, is_static_max):
        """
        设置是否允许
        :param is_static_max:
        :return:
        """
        self.is_static_max = is_static_max

    def set_max_workers(self, num):
        """
        如果线程数是动态的，就允许设置大小
        :param num:
        :return:
        """
        if self.is_static_max:
            self.logger.info("是静态最大线程数,不能对最大线程数进行设置")
        else:
            self.max_workers = num

    def get_max_workers(self):
        return self.max_workers

    def get_localVal(self):
        """
        获取线程独立变量
        :return:
        """
        return self.localVal

    def set_localVal(self, val):
        self.localVal = val

    def set_callback(self, callback):
        self.callback = callback

    def event_set(self):
        """
        设置event 允许wait继续运行 状态为True
        :return:
        """
        self.event.set()

    def event_clear(self):
        """
        设置值为False 不允许运行其他线程
        :return:
        """
        self.event.clear()

    def event_is_set(self):
        """
        返回标志
        :return:
        """
        return self.event.is_set()

    def event_wait(self, timeout=None):
        self.event.wait(timeout)

    def set_thread_name_prefix(self, prefix):
        self.prefix = prefix

    def __init_threading_pool(self, thread_num):
        """
        初始化线程池，创建指定数量的线程池
        :param thread_num:
        :return:
        """
        self.logger.info("初始化工作线程数量为：%d" % thread_num)
        for i in range(int(thread_num)):
            # 新建一个线程
            thread = ThreadManager(self.work_queue,
                                   self.result_queue,
                                   self.thread_queue,
                                   self.logger,
                                   args=self.args,
                                   kwargs=self.kwargs)
            # 线程名
            threadName = "{}{}".format(self.prefix, i + 1)
            if threadName in self.thread_pool_dicts:
                self.logger.warn("线程名%s在线程池中已经存在，跳过" % threadName)
            else:
                # 设置线程名
                thread.setName(threadName)
                self.set_thread_pool_dicts(self.thread_pool_dicts, threadName, thread, self.args, self.kwargs)

    def set_thread_pool_dicts(self, thread_pool_dicts, threadName, thread, args, kwargs):
        """
        这里调用了一个钩子函数 可以在dicts中加入其他参数
        :param thread_pool_dicts: 线程池字典对象
        :param threadName:  线程名
        :param thread:  线程
        :param args:
        :param kwargs:
        :return:
        """
        threadinfo = ThreadInfo()
        threadinfo.set_thread(thread)
        threadinfo.set_kwargs(kwargs)
        threadinfo.set_args(args)
        threadinfo.set_thread_name(threadName)
        if self.callback:
            # 对象使用
            dicts = self.callback(threadinfo)
        else:
            # 继承时使用
            dicts = self.thread_pool_hook(threadinfo)
        if not dicts:
            dicts = {}
        threadinfo.set_dicts(dicts)
        thread_pool_dicts[threadName] = threadinfo

    def thread_pool_hook(self, threadinfo: ThreadInfo) -> dict:
        """
        钩子函数 可以被重写
        主要重写里面的dicts部分
        :return:
        """
        return {}

    def reload_thread(self, thread_name):
        """
        通过线程名重启线程
        :param thread_name:
        :return:
        """
        is_start = False
        if thread_name in self.thread_pool_dicts:
            self.logger.info("{}线程名存在".format(thread_name))
            if self.thread_pool_dicts[thread_name].get_thread().is_alive():
                self.logger.info("{}线程名是存活的".format(thread_name))
                is_start = True
            else:
                self.logger.info("{}线程名是没有存活 需要重启".format(thread_name))

            if not is_start:
                args = self.thread_pool_dicts[thread_name].get_args()
                kwargs = self.thread_pool_dicts[thread_name].get_kwargs()
                self.logger.info("新建或重启线程{}".format(thread_name))
                # 新建一个线程
                thread = ThreadManager(self.work_queue, self.result_queue, self.thread_queue, self.logger, args=args,
                                       kwargs=kwargs)
                # 设置线程名
                thread.setName(thread_name)
                # # 加入线程池
                # self.threadpool.append(thread)
                self.set_thread_pool_dicts(self.thread_pool_dicts, thread_name, thread, args, kwargs)
                thread.start()

    def add_thread(self, thread_num):
        """
        添加线程，thread_num是添加线程的数量
        :param thread_num:
        :return:
        """
        # 判断可开线程的数量
        if self.thread_num < self.max_workers:
            if self.max_workers - self.thread_num < thread_num:
                thread_num = self.max_workers - self.thread_num
        else:
            self.logger.info("线程已达到最大量，不能再新开线程")
            return False
        # 添加新的线程
        count = 0
        num = 0
        while thread_num > count:
            num += 1
            thread = ThreadManager(self.work_queue, self.result_queue, self.thread_queue,
                                   self.logger,
                                   args=self.args,
                                   kwargs=self.kwargs)
            thread_name = "{}{}".format(self.prefix, num)
            if thread_name in self.thread_pool_dicts:
                continue
            thread.setName(thread_name)
            self.logger.info("添加一个线程{}".format(thread_name))
            self.set_thread_pool_dicts(self.thread_pool_dicts, thread_name, thread, self.args, self.kwargs)
            count += 1
        self.get_thread_num()
        return True

    def delete_thread(self, thread_num):
        """
        删除多少个线程，thread_num为删除数量
        :param thread_num:
        :return:
        """
        try:
            i = 0
            for key in list(self.thread_pool_dicts.keys()):
                thread = self.thread_pool_dicts[key].get_thread()
                if not thread.runstatus:
                    # 优先删除处于等待任务的线程
                    thread.thread_delete = True
                    self.stop_thread(thread)
                    del self.thread_pool_dicts[key]
                    i += 1
                elif thread.threadval.thread_delete:
                    # 删除该线程 表示线程池想删除(删除逻辑为我想删除 且程序内部允许删除时删除)
                    thread.thread_delete = True
                    del self.thread_pool_dicts[key]
                    i += 1
                if i >= thread_num:
                    break
        except IndexError:
            pass
        self.get_thread_num()

    def get_thread_num(self):
        self.thread_num = len(self.thread_pool_dicts)
        return self.thread_num

    def set_thread_num(self, thread_num):
        """
        设置当前线程数量 改设置保证了线程数量 多的会被删除,少的会被添加
        :return:
        """
        if thread_num > self.get_thread_num():
            self.add_thread(thread_num - self.thread_num)
        else:
            self.delete_thread(self.get_thread_num() - thread_num)
            # self.threadpool = self.threadpool[:int(thread_num)]
        self.thread_num = thread_num

    def add_super_thread(self, func, thread_name, mode="super", *args, **kwargs):
        """
        添加一个线程，该线程直接由原生线程控制，
        :param func:
        :param thread_name:
        :param args:
        :param kwargs:
        :return:
        """
        thread = ThreadManager(self.work_queue, self.result_queue, self.thread_queue,
                               self.logger,
                               target=func, name=thread_name,
                               args=args,
                               kwargs=kwargs)
        thread.set_mode(mode)
        return thread

    def set_add_especial_thread(self, func, threadname, mode="super", *args, **kwargs):
        """
        添加特殊的线程
        :return:
        """
        # 得到一个线程对象
        thread = self.add_super_thread(func, threadname, mode, *args, **kwargs)
        self.set_thread_pool_dicts(self.especial_thread_pool_dicts, threadname, thread, args, kwargs)
        return thread

    def add_job(self, func, *args, **kwargs):
        """
        将任务放入队列，等待线程池阻塞读取，参数是被执行的函数和函数的参数
        :param func:
        :param args:
        :param kwargs:
        :return:
        """
        self.work_queue.put((func, self.result_queue, args, kwargs))

    def start(self):
        for thread in self.thread_pool_dicts:
            time.sleep(2)
            if self.thread_pool_dicts[thread]:
                thread = self.thread_pool_dicts[thread].get_thread()
                if thread and not thread.is_alive():
                    thread.start()

    def especial_start(self):
        for threadname in self.especial_thread_pool_dicts:
            if self.especial_thread_pool_dicts[threadname]:
                threadinfo = self.especial_thread_pool_dicts[threadname]
                thread = threadinfo.thread
                if thread and not thread.is_alive():
                    self.logger.info("启动线程{}".format(thread.getName()))
                    thread.start()
                else:
                    self.logger.info("启动线程{} 失败".format(thread.getName()))

    def get_now_thread(self):
        """
        获取进程中还存在的线程
        :return:
        """
        # 用来保存当前线程名称
        nowThreadsName = []
        # 获取当前线程名
        now = threading.enumerate()
        for i in now:
            # 保存当前线程名称
            nowThreadsName.append(i.getName())
        return nowThreadsName

    # 防止线程意外挂掉 挂掉线程重启
    def checkThread(self):
        nowThreadsName = self.get_now_thread()
        # 循环线程池的key
        for name in list(self.thread_pool_dicts.keys()):
            # 取出线程对象
            thread = self.thread_pool_dicts[name].get_thread()
            # 如果线程字典为空 代表已被删除
            if name in nowThreadsName and thread.is_alive():
                pass  # 当前某线程名包含在初始化线程组中，可以认为线程仍在运行
            else:
                self.reload_thread(name)  # 重启线程
        # 如果不是静态最大值 线程可以波动
        if not self.is_static_max:
            # 空闲线程
            free_thread = 0
            # 判断线程空闲情况
            for name in list(self.thread_pool_dicts.keys()):
                thread = self.thread_pool_dicts[name].get_thread()
                if not thread.runstatus:
                    # 代表该线程等待工作队列，列入空闲线程
                    free_thread += 1
                elif thread.threadval.thread_delete:
                    free_thread += 1

            # 允许冗余2个工作线程 避免在零界点不停的创建,删除
            if free_thread > 2:
                self.max_workers = self.max_workers - (free_thread - 2)

            # 判断是否需要删除线程
            if len(self.thread_pool_dicts) > self.max_workers:
                self.logger.info("线程池有{},最大线程数量{}，调用删除线程程序".format(len(self.thread_pool_dicts), self.max_workers))
                self.delete_thread(len(self.thread_pool_dicts) - self.max_workers)

        self.logger.info("当前线程数量为{},线程为:{}".format(len(self.thread_pool_dicts), list(self.thread_pool_dicts.keys())))

    def checkThreadRunFinish(self):
        """
        通过等待的方式判断结束，等待是因为任务队列取出任务后在运行，还没放入结果队列
        :return:
        """
        self.logger.info("result_queue is empty:{}".format(self.result_queue.is_empty()))
        self.logger.info("work_queue is empty:{}".format(self.work_queue.is_empty()))
        if self.result_queue.is_empty() and self.work_queue.is_empty():
            time.sleep(10)
            t1 = self.result_queue.is_empty()
            time.sleep(10)
            t2 = self.result_queue.is_empty()
            if t1 and t2:
                if self.work_queue.is_empty():
                    return True
            return False
        return False

    # 下面两个方法是强制退出线程的方法
    def _async_raise(self, tid, exctype):
        """raises the exception, performs cleanup if needed"""
        tid = ctypes.c_long(tid)
        if not inspect.isclass(exctype):
            exctype = type(exctype)
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
        if res == 0:
            raise ValueError("invalid thread id")
        elif res != 1:
            # """if it returns a number greater than one, you're in trouble,
            # and you should call it again with exc=NULL to revert the effect"""
            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
            raise SystemError("PyThreadState_SetAsyncExc failed")

    def stop_thread(self, thread):
        if thread:
            try:
                self.logger.info("thread is {};thread.ident 的值为:{}".format(thread, thread.ident))
                if thread.ident:
                    self._async_raise(thread.ident, SystemExit)
            except ValueError as e:
                self.logger.info("已经运行完毕 不需要删除了")
        else:
            self.logger.info("传入的线程对象为空，理论上不应该出现这种逻辑 结束程序检查逻辑")
            sys.exit(-1)


class ThreadVal(object):
    """
    该对象维护线程函数的变量以及与线程对象间的交互
    """

    def __init__(self):
        # 该线程是否被删除
        self.thread_delete = False
        # 线程初始化参数
        self.args = ()
        self.kwargs = None
        self.result_queue = None
        self.work_queue = None

    def set_args(self, args):
        self.args = args

    def set_kwargs(self, kwargs):
        self.kwargs = kwargs

    def set_result_queue(self, result_queue):
        self.result_queue = result_queue

    def get_result_queue(self):
        return self.result_queue

    def set_work_queue(self, work_queue):
        self.work_queue = work_queue

    def get_work_queue(self):
        return self.work_queue


class ThreadManager(Thread):
    """
    定义线程类，继承threading.Thread
    该类主要实现线程的运行
    """

    def __init__(self, work_queue=None, result_queue=None, thread_queue=None, logger=None, group=None, target=None,
                 name=None, args=(), kwargs=None, daemon=True, mode="default"):
        if mode == "default":
            if work_queue is None or result_queue is None or thread_queue is None:
                raise Exception("work_queue result_queue thread_queue 必须存在")
        if args is None:
            args = ()
        # 线程本身的参数
        Thread.__init__(self, group, target, name,
                        args, kwargs, daemon=daemon)
        if kwargs is None:
            kwargs = {}
        if logger:
            self.logger = logger
        else:
            self.logger = get_streamlogger()
        self.target = target
        self.kwargs = kwargs
        self.args = args
        # 接受两个队列 一个处理结果 一个发送任务
        self.work_queue = work_queue
        self.result_queue = result_queue
        # 线程队列
        self.thread_queue = thread_queue
        # 主线程退出  该线程也会退出
        self.daemon = daemon
        # 一个标识 如果为False代表线程运行完毕等待新的任务到来,否则表示任务正在运行
        # 这个标志是为了检查程序在运行中还是等待状态
        self.runstatus = False
        # 设置该标志是为了表示是否愿意被重新启动，因为有些线程只需要运行一次
        # 而有的需要长期运行且有一定的重启机制(目前暂未启动该标志)
        self.is_restart = True
        # 为了兼容普通线程类模式  "default" 自己写的run 方法  "super": 父类的run方法
        self.mode = mode
        # 是否可以被删除 True 线程可以停止，False 不可以被停止 有时候我们需要删除一批线程 但是 线程不能中间
        # 状态被删除 故需要设置该标志
        self.thread_delete = False
        # 线程内部的一个类对象 每个线程维护一个保证里面的数据线程安全，以后为每个线程提供服务
        self.threadval = ThreadVal()
        self.threadval.set_args(self.args)
        self.threadval.set_kwargs(self.kwargs)

    def get_runstatus(self):
        return self.runstatus

    def set_runstatus(self, runstatus):
        self.runstatus = runstatus

    def get_is_restart(self):
        return self.is_restart

    def set_is_restart(self, is_restart):
        self.is_restart = is_restart

    def set_mode(self, mode):
        """
        设置运行模式 super or default
        super 将方法直接传入target
        default 使用自己封装的run
        :param mode:
        :return:
        """
        self.mode = mode

    def get_mode(self):
        return self.mode

    def run(self):
        if self.mode == "default":
            self.run_default()
        elif self.mode == "mysuper":
            self.run_arg()
        elif self.mode == "super":
            try:
                # 父类的运行程序，相当于原生支持
                super().run()
            except:
                self.logger.info(traceback.format_exc())
        else:
            raise Exception("运行模式错误 请检查 default or super")

    def run_arg(self):
        try:
            if self.target:
                self.threadval.set_result_queue(self.result_queue)
                self.threadval.set_work_queue(self.work_queue)
                self.target(self.threadval, *self.args, **self.kwargs)
        finally:
            del self.target, self.args, self.kwargs

    def run_default(self):
        """
        启动线程,默认的运行程序，
        :return:
        """
        while True:
            try:
                if self.thread_delete and self.threadval.thread_delete:
                    print("主动停止线程{},因为两个信号量都为True".format(self.getName()))
                    break
                # 该标志标识着是否取得队列值并开始工作
                self.runstatus = False
                target, result_queue, args, kwargs = self.work_queue.get()
                self.runstatus = True  # 代表取到工作队列的值
                # 更新字典 如果键相同 以初始化线程的键的值有效 所以尽量不要与初始化字典冲突
                kwargs.update(self.kwargs)
                # 初始化线程的元组参数拼接在后
                args = args + self.args
                #
                self.threadval.set_result_queue(result_queue)
                self.threadval.set_work_queue(self.work_queue)
                target(self.threadval, *args, **kwargs)
                self.work_queue.task_done()
            except UnicodeDecodeError as e:
                self.logger.error(traceback.format_exc())
            except SystemExit as e:
                if e.args == (-1,):
                    self.logger.error("由逻辑判断主动退出当前线程")
                    break
                else:
                    self.logger.error("其他SystemExit错误，检查逻辑")



import abc
import asyncio
import sys
from concurrent.futures import ProcessPoolExecutor
import gzip

import multiprocessing
from pathlib import Path

from io import BytesIO
import time
from typing import Awaitable, Callable, Any, Generator, List, Literal, Union

from hdfs import InsecureClient


from re_common.v2.baselibrary.tools.resume_tracker import ResumeTracker


_pool = None
_loop = None


class HDFSBaseProcessor(abc.ABC):
    def __init__(
        self,
        hdfs_dir: str,
        hdfs_url: str = "http://VIP-DC-MASTER-2:9870",
        hdfs_user: str = "root",
        db_path: Union[str, Path] = "processed_files.db",
        concurrency: int = 50,
        batch_size: int = 50,
        encoding: str = "utf-8",
        read_mode: Literal["all", "stream"] = "all",
        retries: int = 3,
        pool_factory: Callable[[], Awaitable[Any]] = None,
        max_processes: int = None,  # 添加多进程支持
        result_dir: str = None,
    ):
        self.hdfs_dir = hdfs_dir
        self.hdfs_url = hdfs_url
        self.hdfs_user = hdfs_user
        self.tracker = ResumeTracker(db_path)
        self.concurrency = concurrency
        self.batch_size = batch_size
        self.encoding = encoding
        self.read_mode = read_mode
        self.retries = retries
        self.result_dir = result_dir
        self.pool_factory = pool_factory
        self.max_processes = max_processes or max(multiprocessing.cpu_count() - 1, 1)  # 默认使用CPU核心数-1
        self._client = None

        self.tracker.init_db()

    @property
    def client(self):
        if self._client is None:
            self._client = InsecureClient(self.hdfs_url, user=self.hdfs_user)
        return self._client

    async def _get_pool(self):
        if self.pool_factory is None:
            return None
        global _pool
        if _pool is None:
            _pool = await self.pool_factory()
        return _pool

    def _list_gz_files(self) -> List[str]:
        """列出 HDFS 目录中的所有 gzip 文件"""
        return [
            f"{self.hdfs_dir}/{file[0]}"
            for file in self.client.list(self.hdfs_dir, status=True)
            if file[0].endswith(".gz")
        ]

    def _count_total_lines(self, gz_file_path: str) -> int:
        with self.client.read(gz_file_path) as hdfs_file:
            with gzip.GzipFile(fileobj=hdfs_file) as gz:
                return sum(1 for _ in gz)

    def _batch_read_gz_stream(self, gz_file_path: str) -> Generator[List[str], Any, None]:
        """流式读取gz文件，分批yield返回"""
        with self.client.read(gz_file_path) as hdfs_file:
            with gzip.GzipFile(fileobj=hdfs_file) as gz:
                while True:
                    lines = []
                    for _ in range(self.batch_size):
                        try:
                            line = next(gz)
                            if line.strip():  # 移除空行
                                lines.append(line.decode(self.encoding))  # 解码
                        except StopIteration:  # 文件已读完
                            break
                    if not lines:
                        break
                    yield lines

    def _batch_read_gz_all(self, gz_file_path: str) -> List[List[str]]:
        """一次读取gz文件全部内容，二维数组批量返回"""
        with self.client.read(gz_file_path) as reader:  # 以二进制模式读取
            compressed_data = reader.read()  # 读取压缩数据
            with gzip.GzipFile(fileobj=BytesIO(compressed_data)) as gz_file:  # 解压缩
                content = gz_file.read().decode(self.encoding)  # 解码为字符串
                print(f"文件读取成功: {gz_file_path}")
                lines = [i for i in content.split("\n") if i.strip()]
                batch_lines = [lines[i : i + self.batch_size] for i in range(0, len(lines), self.batch_size)]
                return batch_lines

    def _batch_read_gz(self, gz_file_path: str):
        # 这里根据不同的配置选用不同的读取文件的方法
        if self.read_mode == "stream":
            return self._batch_read_gz_stream(gz_file_path)
        else:
            return self._batch_read_gz_all(gz_file_path)

    def _generate_write_data(self, results):
        for res in results:
            yield str(res) + "\n"

    def _print_progress(self, file_path, processed_lines, total_lines, start_time, processing_start_time):
        elapsed_time = time.perf_counter() - start_time  # 已用时间
        processing_time = time.perf_counter() - processing_start_time  # 本次处理时间
        avg_processing_time = (
            (elapsed_time * 1000) / processed_lines if processed_lines > 0 else float("inf")
        )  # 平均每条数据的处理时间（毫秒）
        # 估算剩余时间
        remaining_time = (
            ((avg_processing_time / 1000) * (total_lines - processed_lines)) if processed_lines > 0 else float("inf")
        )
        # 显示进度信息
        print(
            f"文件: {file_path} 总进度: {processed_lines}/{total_lines} 行 | "
            f"已用时间: {elapsed_time:.2f}秒 | 本次处理时间: {processing_time:.2f}秒 | "
            f"预估剩余时间: {remaining_time:.2f}秒 | 平均每条处理时间: {avg_processing_time:.2f}毫秒"
        )

    def _print_final_progress(self, file_path, processed_lines, total_lines, start_time):
        final_elapsed_time = time.perf_counter() - start_time  # 最终已用时间
        print(
            f"文件: {file_path} 处理完成 | 总进度: {processed_lines}/{total_lines} 行 | "
            f"总已用时间: {final_elapsed_time:.2f}秒 | "
            f"平均每条处理时间: {(final_elapsed_time * 1000) / processed_lines:.2f}毫秒"
            if processed_lines > 0
            else "处理无数据"
        )

    @abc.abstractmethod
    async def _process_file(self, hdfs_file_path, process_func):
        pass

    async def _retry_process_file(self, hdfs_file_path, process_func):
        """带重试机制的文件处理"""
        retry_count = 0
        while retry_count < self.retries:
            try:
                if self.tracker.is_processed(hdfs_file_path):
                    print(f"跳过已处理文件: {hdfs_file_path}")
                    return True
                await self._process_file(hdfs_file_path, process_func)
                self.tracker.mark_processed(hdfs_file_path)  # 标记文件已处理
                return True  # 成功处理后退出
            except Exception as e:
                retry_count += 1
                print(f"处理文件 {hdfs_file_path} 时发生错误: {e}，正在重试 {retry_count}/{self.retries}")
                await asyncio.sleep(2**retry_count)
        print(f"处理文件 {hdfs_file_path} 失败，达到重试上限")
        return False

    def _process_file_wrapper(self, args):
        """为多进程执行准备的同步包装函数"""
        hdfs_file_path, process_func = args
        if sys.platform == "win32":
            loop = asyncio.get_event_loop()
            return loop.run_until_complete(self._retry_process_file(hdfs_file_path, process_func))
        else:
            global _loop
            if _loop is None:
                _loop = asyncio.new_event_loop()
                asyncio.set_event_loop(_loop)
            return _loop.run_until_complete(self._retry_process_file(hdfs_file_path, process_func))

    async def _run_multi_process(self, gz_files, process_func):
        """多进程并发运行文件处理任务"""
        args_list = [(file_path, process_func) for file_path in gz_files]
        with ProcessPoolExecutor(max_workers=self.max_processes) as executor:
            # return executor.map(self._process_file_wrapper, args_list)
            loop = asyncio.get_running_loop()
            self._client = None  # 避免连接对象无法序列化导致卡死
            tasks = [loop.run_in_executor(executor, self._process_file_wrapper, args) for args in args_list]
            results = await asyncio.gather(*tasks)

        if all(results):
            # 处理完成后清理断点记录
            self.tracker.clear_processed_items()
            print(f"已清空断点记录: {self.tracker.db_path}")
            return results
        else:
            raise Exception("部分或全部文件处理失败")

    @abc.abstractmethod
    async def map(self, process_func: Callable[[Any, Any], Awaitable[Any]]) -> None:
        pass



import numpy as np
import pandas as pd
from pandas import DatetimeIndex, DataFrame

"""
https://www.pypandas.cn/docs/getting_started/10min.html#%E6%9F%A5%E7%9C%8B%E6%95%B0%E6%8D%AE
 Series（一维数据）;带标签的一维同构数组
 DataFrame（二维数据）;带标签的，大小可变的，二维异构表格;index（行）或 columns（列）
 DataFrame 是 Series 的容器，Series 则是标量的容器
 NumPy 数组只有一种数据类型，DataFrame 每列的数据类型各不相同
"""


class BasePandas(object):

    def __init__(self):
        pass

    def create_null_dataframe(self):
        """
        创建空的dataframe
        :return: type:DataFrame
        """
        df = pd.DataFrame()
        return df

    def create_series_for_list(self, list):
        """
        用值列表生成 Series 时，Pandas 默认自动生成整数索引
        pd.Series([1, 3, 5, np.nan, 6, 8])
        :return: type:Series
        """
        s = pd.Series(list)
        # print(type(s))
        return s

    def create_time_index(self, datastring, periods):
        """
        创建行标
        含日期时间索引与标签的 NumPy 的数组
        :param datastring: '20130101'
        :param periods: 6
        :return: type:DatetimeIndex
        """
        dates = pd.date_range(datastring, periods=periods)
        return dates

    def create_ndarray(self, index, columns):
        """
        产生指定行列的随机数据
        :param index: 行  6
        :param columns: 列 4
        :return:  type:ndarray
        """
        return np.random.randn(index, columns)

    def create_time_dataform(self, data, dates: DatetimeIndex, columns=list('ABCD')):
        return pd.DataFrame(data, index=dates, columns=columns)

    def dicts_to_dataform(self, dicts):
        """
        字典转二维数据
        {'A': 1.,
        'B': pd.Timestamp('20130102'),
        'C': pd.Series(1, index=list(range(4)), dtype='float32'),
        'D': np.array([3] * 4, dtype='int32'),
        'E': pd.Categorical(["test", "train", "test", "train"]),
        'F': 'foo'}
        :param dicts:
        :return:
        """
        df = pd.DataFrame(dicts)
        return df

    def dtypes(self, df: DataFrame):
        """
        DataFrame 的列的数据类型
        :return:
        """
        return df.dtypes

    def head(self, df, num=5):
        """
        查看前几条数据（默认前五条）
        :param num:
        :return:
        """
        return df.head(num)

    def tail(self, df, num=5):
        """
        查看后几条数据（默认后五条）
        :param num:
        :return:
        """
        return df.tail(num)

    def index(self, df):
        """
        显示索引
        :param df:
        :return:
        """
        return df.index

    def columns(self, df):
        """
        显示列名
        :param df:
        :return:
        """
        return df.columns

    def dataform_to_numpy(self, df):
        """
        输出底层数据的 NumPy 对象。
        注意，DataFrame 的列由多种数据类型组成时，该操作耗费系统资源较大，
        输出不包含行索引和列标签
        :return:
        """
        return df.to_numpy()

    def describe(self, df):
        """
        可以快速查看数据的统计摘要：
        :param df:
        :return:
        """
        return df.describe()

    def df_T(self, df):
        """
        转置数据
        :param df:
        :return:
        """
        return df.T

    def sort_index(self, df):
        """
        按轴排序
        :return:
        """
        return df.sort_index(axis=1, ascending=False)

    def sort_values(self, df):
        """
        按值排序
        :param df:
        :return:
        """
        return df.sort_values(by='B')

    def get_series(self, df):
        """
        获取单列数据 等于 df.A
        :param df:
        :return:
        """
        return df["A"]

    def get_spilt(self, df):
        """
        切片行 或者 df['20130102':'20130104']
        :return:
        """
        return df[0:3]

    def get_loc(self, df, dates: DatetimeIndex):
        """
        标签提取一行数据
        :return:
        """
        return df.loc[dates[0]]

    def get_many_loc(self, df):
        """
        用标签选择多列数据
        :return:
        """
        return df.loc[:, ['A', 'B']]

    def get_many_loc_index(self, df):
        """
        用标签切片，包含行与列结束点
        :param df:
        :return:
        """
        return df.loc['20130102':'20130104', ['A', 'B']]

    def get_onedata(self, df, dates):
        """
        提取标量值
        快速访问标量，与上述方法等效：df.at[dates[0], 'A']
        :return:
        """
        return df.loc[dates[0], 'A']

    def get_index(self, df):
        """
        获取行 用整数位置选择
        :return:
        """
        return df.iloc[3]

    def get_qiepian(self, df):
        """
        3:5 为行 0:2列
        用整数列表按位置切片
        df.iloc[[1, 2, 4], [0, 2]]
        显式整行切片
         df.iloc[1:3, :]
         显式整列切片
         df.iloc[:, 1:3]
         显式提取值
         df.iloc[1, 1]
         快速访问标量，与上述方法等效：
         df.iat[1, 1]
        :param df:
        :return:
        """
        return df.iloc[3:5, 0:2]

    def select_data(self, df):
        """
        用单列的值选择行数据
        选择 DataFrame 里满足条件的值：
        df[df > 0]
        :param df:
        :return:
        """
        return df[df.A > 0]

    def copy(self, df):
        df2 = df.copy()
        return df2

    def add_col(self, df):
        """
        添加列
        :return:
        """
        df['E'] = ['one', 'one', 'two', 'three', 'four', 'three']
        return df

    def isin(self, df):
        """
        用 isin() 筛选 行
        :return:
        """
        return df[df['E'].isin(['two', 'four'])]

    def set_value(self, df):
        """
        按标签赋值
        df.at[dates[0], 'A'] = 0
        按位置赋值：
         df.iat[0, 1] = 0
         按 NumPy 数组赋值：
         df.loc[:, 'D'] = np.array([5] * len(df))
         用 where 条件赋值：
         df2 = df.copy()
         df2[df2 > 0] = -df2
         Pandas 主要用 np.nan 表示缺失数据
        :param df:
        :return:
        """
        s1 = pd.Series([1, 2, 3, 4, 5, 6], index=pd.date_range('20130102', periods=6))
        df['F'] = s1

    def reindex(self, df, dates):
        """
        重建索引（reindex）可以更改、添加、删除指定轴的索引，并返回数据副本，即不更改原数据。
        :param df:
        :return:
        """
        df1 = df.reindex(index=dates[0:4], columns=list(df.columns) + ['E'])
        df1.loc[dates[0]:dates[1], 'E'] = 1

    def dropna(self, df):
        """
        删除所有含缺失值的行：
        :param df:
        :return:
        """
        return df.dropna(how='any')

    def fillna(self, df):
        """
        填充缺失值
        :param df:
        :return:
        """
        return df.fillna(value=5)

    def isna(self,df):
        """
        提取 nan 值的布尔掩码
        :param df:
        :return:
        """
        return pd.isna(df)



import gzip
from io import BytesIO
from typing import List

from hdfs import InsecureClient

from re_common.v2.baselibrary.tools.data_processer.base import BaseFileWriter


class HDFSFileWriter(BaseFileWriter):
    def __init__(self, file_path: str, hdfs_url: str, hdfs_user: str, *args, **kwargs):
        super().__init__(file_path, *args, **kwargs)
        self.client = InsecureClient(hdfs_url, user=hdfs_user)

    def write_lines(self, lines: List[str], file_path: str = None):
        if file_path is None:
            file_path = self.file_path
        data = "\n".join(lines).encode(self.encoding)
        if self.compress:
            buf = BytesIO()
            with gzip.GzipFile(fileobj=buf, mode="wb") as gz:
                gz.write(data)
            buf.seek(0)
            self.client.write(file_path, data=buf, overwrite=self.overwrite)
        else:
            self.client.write(file_path, data=data, overwrite=self.overwrite)


class LocalFileWriter(BaseFileWriter):
    def write_lines(self, lines: List[str], file_path: str, compress: bool = True, encoding="utf-8"):
        if compress:
            with gzip.open(file_path, 'wt', encoding=encoding) as f:
                for line in lines:
                    f.write(f"{line}\n")
        else:
            with open(file_path, 'w', encoding=encoding) as f:
                for line in lines:
                    f.write(f"{line}\n")



"""
多文件合并到一个文件
方法一：按行读取写入
方法二: 按块读取写入

"""

###########################################
# 同项目调用基础包
import os
import sys

filepath = os.path.abspath(__file__)
pathlist = filepath.split(os.sep)
pathlist = pathlist[:-4]
TopPath = os.sep.join(pathlist)
sys.path.insert(0, TopPath)
############################################

import time

from re_common.baselibrary.utils.basedir import BaseDir
from re_common.baselibrary.utils.basefile import BaseFile


class MergeFile(object):
    def __init__(self):
        """
        :param src: 输入文件夹路径
        :param dst: 输出文件夹路径
        """
        self.outPathFile = None
        self.a_num = 1000
        self.b_num = 20000

        self.a_size = 1024 * 1024 * 1024
        self.block_size = 2 * 1024 * 1024 * 1024

    def get_outpathfile(self, dst):
        BaseDir.create_dir(dst)
        now_time = time.strftime('%Y%m%d', time.localtime())
        self.filename = "{}.big_json".format(now_time)
        self.outPathFile = BaseFile.get_new_path(dst, self.filename)
        return self.outPathFile

    def line_all_2_one(self, src, dst, source_size=None, dst_size=None, a_num=None, b_num=None):
        """
        :param src: 输入路径
        :param dst: 输出路径
        :param source_size: 输入文件大于source_size，就直接复制过去
        :param dst_size:  指定输出文件的最大值
        :param a_num:  打印条数的取值（默认1000条输出一次信息）
        :param b_num:  打印路径的取值（默认20000条输出一次信息）
        :return:
        """
        # 程序开始时间
        StartTime = time.time()
        if a_num is None:
            a_num = self.a_num
        if b_num is None:
            b_num = self.b_num
        count_all = 0
        dst_count = 0
        src_file_num = 0
        src_file_location = 0
        if self.outPathFile is None:
            self.get_outpathfile(dst)
        for files in BaseDir.get_dir_all_files(src):
            src_file_num += 1
        for files in BaseDir.get_dir_all_files(src):
            # 处理每个文件的开始时间
            start_time = time.time()
            src_file_location += 1
            print("next source_path:" + files)
            if source_size is not None:
                if BaseFile.get_file_size(files) >= source_size:
                    out_path = BaseFile.get_new_path(dst, BaseFile.get_file_name(files))
                    BaseFile.copy_file_to_file(files, out_path)
                    print("process_src:%d/%d, Time total:%s, source_path: %s , dst_path:%s" % (
                    src_file_location, src_file_num, (repr(time.time() - start_time)), files, out_path))
                    continue
            count = 0
            for line in BaseFile.read_file_r_mode_yield(files):
                count += 1
                count_all += 1
                dst_count += 1
                line = line + '\n'
                BaseFile.single_add_file(self.outPathFile, line)
                if count % a_num == 1:
                    print("process_src:%d/%d, Time total:%s, source_count: %d, dst_count: %d, allcount: %d" % (
                    src_file_location, src_file_num, (repr(time.time() - start_time)), count, dst_count, count_all))
                if count % b_num == 1:
                    print("source_path: %s , dst_path:%s" % (files, self.outPathFile))
                if dst_size is not None:
                    if BaseFile.get_file_size(self.outPathFile) >= dst_size:
                        print("cut dst file, now:source_count: %d,dst_count: %d, allcount: %d" % (
                        count, dst_count, count_all))
                        dst_count = 0
                        self.outPathFile = BaseFile.change_file(self.outPathFile, size=dst_size)
                        print("new dst_path: %s" % self.outPathFile)

            print("finish one file;Time total:%s, process_src:%d/%d, source_count: %d, dst_count: %d, allcount: %d" % (
            repr(time.time() - start_time), src_file_location, src_file_num, count, dst_count, count_all))
        # 程序结束花费时间时间
        print('\nProce Over\nTime total:' + repr(time.time() - StartTime) + '\n')

    def block_all_2_one(self, src, dst, source_size=None, dst_size=None, block_size=None, a_size=None):
        """
        按块读取小文件，并写入文件
        :param src: 输入路径
        :param dst: 输出路径
        :param source_size: 输入文件大于source_size，就直接复制过去
        :param dst_size: 指定输出文件的最大值（但是需要操作完一个输入文件后判断大小）
        :param block_size: 按块读取的值
        :param a_size: 打印路径的取值（默认1G大小输出一次信息）
        :return:
        """
        # 程序开始时间
        StartTime = time.time()
        src_file_num = 0
        src_all_size = 0
        src_file_location = 0
        if a_size is None:
            a_size = self.a_size
        if block_size is None:
            block_size = self.block_size

        if self.outPathFile is None:
            self.get_outpathfile(dst)
        for files in BaseDir.get_dir_all_files(src):
            src_file_num += 1
        for files in BaseDir.get_dir_all_files(src):
            # 处理每个文件的开始时间
            src_all_size += BaseFile.get_file_size(files)
            dst_file_size = 0
            start_time = time.time()
            src_file_location += 1
            print("next source_path:" + files)
            if source_size is not None:
                if BaseFile.get_file_size(files) >= source_size:
                    out_path = BaseFile.get_new_path(dst, BaseFile.get_file_name(files))
                    BaseFile.copy_file_to_file(files, out_path)
                    print("process_src:%d/%d, Time total:%.2f/%.2f, source_path: %s , dst_path:%s" % (
                    src_file_location, src_file_num, (time.time() - start_time), (time.time() - StartTime), files,
                    out_path))
                    continue
            for block in BaseFile.read_file_rb_block(files, BLOCK_SIZE=block_size):
                BaseFile.single_write_ab_file(block, self.outPathFile)
            if BaseFile.get_file_size(self.outPathFile) % a_size == 1:
                print("process_src:%d/%d, dst_size:%d, src_size:%d, src_all_size:%d, Time total:%.2f/%.2f" % (
                src_file_location, src_file_num, dst_file_size, BaseFile.get_file_size(files), src_all_size,
                (time.time() - start_time), (time.time() - StartTime)))
            dst_file_size = BaseFile.get_file_size(self.outPathFile)
            print(
                "finish one file;process_src:%d/%d, dst_size:%d, src_size:%d, src_all_size:%d, Time total:%.2f/%.2f" % (
                src_file_location, src_file_num, dst_file_size, BaseFile.get_file_size(files), src_all_size,
                (time.time() - start_time), (time.time() - StartTime)))
            if dst_size is not None:
                if BaseFile.get_file_size(self.outPathFile) >= dst_size:
                    self.outPathFile = BaseFile.change_file(self.outPathFile, size=dst_size)
                    print("new dst_path: %s" % self.outPathFile)
                    dst_file_size = 0
            # 程序结束花费时间时间
        print('\nProce Over\nTime total:' + repr(time.time() - StartTime) + '\n')

# if __name__ == '__main__':
#     old_path = r'E:\work\pnasjournal\big_json\20200116'
#     new_path = r'E:\work\pnasjournal\big_json\new'
#     a = MergeFile()
#     # a.line_all_2_one(old_path,new_path)
#     a.block_all_2_one(old_path,new_path,dst_size=1024*1024*10)



from re_common.baselibrary.tools.myparsel import MParsel

if __name__ == '__main__':
    htmlText = r'''
    <body>
        <div class="book">111</div>
        <div class="journal_name">222</div>
        <div class="title">333</div>
        <a class ="link">link1</a>
        <a class ="link">link2</a>
        <a class ="link">link3</a>
    </body>
'''
    mc = MParsel()
    css_selector = {
        'book':'div[class="book"]::text',
        'journal_name':'div[class="journal_name"]::text',
        'title':'div[class="title"]::text',
        'link' :'a[class*="link"]::text'

    }
    xpath_selector = {
        'book': "//div[@class='book']/text()",
    }
    bools,new_dict = mc.css_parsel_html(htmlText,css_selector=css_selector)
    print(new_dict)
    bools, new_dict = mc.xpath_parsel_html(htmlText, xpath_selector=xpath_selector)
    print(new_dict)


class BaseEsDSL(object):
    """
    将条件转换成dsl语句
    """

    def __init__(self):
        self.size = 10000
        self.dsl = {}
        # 字段列表
        self.source = []

    def set_size(self, size):
        """
        设置返回大小一般为10000
        :param size:
        :return:
        """
        self.size = size
        return self

    def set_source(self, source):
        """
        字段列表
        例如 ["keyid", "lngid", "title", "author", "pub_year", "source_type"]
        :param source:
        :return:
        """
        self.source = source

    def term(self, key, values, is_keyword=True):
        if is_keyword:
            key = key + ".keyword"
        return {"term": {key, values}}

    def terms(self, key, lists: list):
        """
        terms：查询某个字段里含有多个关键词的文档
        :param key:
        :param lists:
        :return:
        """
        return {"terms": {key: lists}}

    def match(self, key, values, is_fuzziness=True):
        """
        注意 match 和 match_phrase 之间的关系是should
        :param key:
        :param values:
        :param is_fuzziness:
        :return:
        """
        if is_fuzziness:
            values = {"query": values, "fuzziness": "1"}
        return {"match": {key, values}}

    def match_phrase(self, key, values):
        """
         注意 match 和 match_phrase 之间的关系是should
        :return:
        """
        return {"match_phrase": {key, values}}

    def range(self, key, dicts):
        """
        key 比如 pub_year
        传入一个字典 {"gte": "2011", "lte": "2021"}
        :param key:
        :param dicts:
        :return:
        """
        return {"range": {key: dicts}}

    def get_must(self):
        """
        获取一个must的结构
        :return:
        """
        return {"bool": {"must": []}}

    def set_query(self, dicts):
        self.dsl["query"] = dicts

    def build(self):
        self.dsl["size"] = self.size
        self.dsl["_source"] = self.source
        self.set_query(self.get_must())



# coding=utf-8
# pip v10.0.0以上版本需要导入下面的包
from subprocess import call

from pip._internal.utils.misc import get_installed_distributions

'''
用于升级python的扩展包一次性升级所有另外提供命令行方式
pip install pip-review
pip-review --local --interactive

环境迁移
pip freeze > requirements.txt

pip uninstall -ry requirements.txt

pip install -r requirements.txt
'''
for dist in get_installed_distributions():
    call("pip install --upgrade " + dist.project_name, shell=True)




import requests


# 注意；basic_json是一个字典，存储一些平台基本信息，样例如下：
# {"user_name":"fuyu",     # 用户名
#  "token":"cce8747e20881dfasdf66b0511cdc9fb2", # 个人令牌
#  "project_name":"project_name",  # 项目名称
#  "task_name":"project_name" # 任务名称
#}

# 用于获取项目代码和任务代码
def get_dolphin_scheduler_code(basic_json):
    # 设置请求头，包含认证token
    headers = {
        "token": basic_json['token']
    }

    # 发送GET请求，获取项目列表
    response = requests.get(url='http://192.168.98.21:12345/cqvip/projects/list',
                            headers=headers)

    # 解析响应中的JSON数据，获取项目列表
    project_list = response.json()['data']

    # 遍历项目列表，查找与指定项目名称匹配的项目
    projectCode=-1
    for i in project_list:
        if i['name'] == basic_json['project_name']:
            projectCode = i['code']  # 获取匹配项目的代码
            break

    # 检查是否找到了匹配的项目代码
    if projectCode!=-1:
        # 发送GET请求，获取指定项目下的任务定义列表
        response = requests.get(
            url='http://192.168.98.21:12345/cqvip/projects/' + str(projectCode) + '/process-definition/list',
            headers=headers)

        # 解析响应中的JSON数据，获取任务定义列表
        task_list = response.json()['data']

        # 遍历任务定义列表，查找与指定任务名称匹配的任务
        task_code=-1
        for task in task_list:
            if task['processDefinition']['name'] == basic_json['task_name']:
                task_code = task['processDefinition']['code']  # 获取匹配任务的代码
                break
        if task_code==-1:
            raise Exception('未找到', basic_json['task_name'], '任务')
    else:
        raise Exception('未找到',basic_json['project_name'],'项目')
    return (str(projectCode),str(task_code))

# 定义一个函数dolphinscheduler_run，用于在DolphinScheduler中运行指定的任务
def dolphinscheduler_run(basic_json):
    # 设置请求头，包含认证token
    headers = {
        "token": basic_json['token']
    }
    code=get_dolphin_scheduler_code(basic_json)
    # 准备请求数据，用于启动任务实例
    data = {
        "projectCode": code[0],  # 项目代码
        "processDefinitionCode": code[1],  # 任务定义代码
        "tenantCode": basic_json['user_name'],  # 用户名称
        "scheduleTime": "",  # 调度时间（留空表示立即执行）
        "failureStrategy": "END",  # 失败策略（END表示失败后结束）
        "warningType": "NONE",  # 警告类型（NONE表示不发送警告）
        "processInstancePriority": "MEDIUM",  # 任务实例优先级（MEDIUM表示中等）
    }
    # 发送POST请求，启动任务实例
    r = requests.post(url='http://192.168.98.21:12345/cqvip/projects/' + str(
        code[0]) + '/executors/start-process-instance',
                      headers=headers, params=data)

    # 打印响应内容，通常包含任务启动的结果信息
    print(r.text)

## 修改 DolphinScheduler 平台上任务的上下线状态
def alter_dolphinscheduler_state(basic_json, releaseState):
    """
    参数:
    - basic_json (dict): 包含用户认证信息的字典，必须包含 'token'、'user_name'，'project_name'和'spark_test' 字段。
    - releaseState (str): 任务的目标状态，'ONLINE' 表示上线，'OFFLINE' 表示下线。
    """

    # 设置请求头，包含认证 token
    headers = {
        "token": basic_json['token']
    }

    # 获取任务代码，假设 get_dolphin_scheduler_code 是一个函数，返回项目代码和任务代码
    code = get_dolphin_scheduler_code(basic_json)  # code[0] 是项目代码，code[1] 是任务代码

    # 构建 API URL，用于修改任务的上下线状态
    API_URL = "http://192.168.98.21:12345/cqvip/projects/" + code[0] + "/process-definition/" + code[1] + "/release"

    # 准备请求数据，包含任务的目标状态
    data = {
        "releaseState": releaseState  # 'OFFLINE' 表示下线状态，'ONLINE' 表示上线状态
    }

    # 发送 POST 请求，修改任务的上下线状态
    # - url: API 地址
    # - headers: 请求头，包含认证信息
    # - params: 请求参数，包含任务的目标状态
    r = requests.post(url=API_URL, headers=headers, params=data)

    # 打印 API 响应内容，通常包含修改状态的结果信息
    print(r.text)


# 修改 DolphinScheduler 平台上的文件内容
def alter_dolphinscheduler_file(basic_json, local_file_path, dolphin_scheduler_file_path):
    """
    参数:
    - basic_json (dict): 包含用户认证信息的字典，必须包含 'token' 和 'user_name' 字段。
    - local_file_path (str): 本地文件的路径，用于读取更新后的内容。
    - dolphin_scheduler_file_path (str): 需要修改的文件在 DolphinScheduler 中的路径（相对路径）。
    """

    # DolphinScheduler 的 API 地址，用于更新文件内容
    URL = "http://192.168.98.21:12345/cqvip/resources/update-content"

    # 打开本地文件，读取文件内容
    with open(local_file_path, 'r', encoding='utf-8') as f:
        content_text = f.read()  # 读取文件的全部内容

    # 设置请求头，包含认证 token
    headers = {
        "token": basic_json['token']
    }

    # 准备请求数据，包含文件内容、用户信息和文件路径
    data = {
        "content": content_text,  # 更新后的文件内容
        "tenantCode": basic_json['user_name'],  # 用户名称
        "fullName": "dolphinscheduler/" + basic_json['user_name'] + "/resources" + dolphin_scheduler_file_path
        # 文件在 DolphinScheduler 中的完整路径
    }

    # 发送 PUT 请求，更新 DolphinScheduler 中的文件内容
    # - url: API 地址
    # - headers: 请求头，包含认证信息
    # - data: 请求数据，包含文件内容和路径信息
    r = requests.put(url=URL, headers=headers, data=data)

    # 打印 API 响应内容，通常包含更新结果信息
    print(r.text)


# 将本地文件上传到 DolphinScheduler 的资源目录中
def upload_dolphinscheduler_file(basic_json, local_file_path, file_name, dolphin_scheduler_file_path):
    """
    参数:
    - basic_json (dict): 包含用户认证信息的字典，必须包含 'token' 和 'user_name' 字段。
    - local_file_path (str): 本地文件的路径。
    - file_name (str): 上传到 DolphinScheduler 后的文件名。
    - dolphin_scheduler_file_path (str): 文件在 DolphinScheduler 中的目标路径（相对路径）。
    """

    # DolphinScheduler 的 API 地址
    API_URL = "http://192.168.98.21:12345/cqvip/resources"

    # 设置请求头，包含认证 token
    headers = {
        "token": basic_json['token'],
    }

    # 准备请求数据，包含文件类型、文件名、描述和目标路径
    data = {
        "type": "FILE",  # 文件类型
        "name": file_name,  # 上传后的文件名
        "description": "undefined",  # 文件描述（此处为默认值）
        "currentDir": "dolphinscheduler/" + basic_json['user_name'] + "/resources" + dolphin_scheduler_file_path  # 目标路径
    }

    # 发送 POST 请求，上传文件
    # - url: API 地址
    # - headers: 请求头，包含认证信息
    # - files: 上传的文件内容，以二进制形式读取本地文件
    # - params: 其他请求参数，包含文件信息和目标路径
    r = requests.post(url=API_URL, headers=headers, files={'file': open(local_file_path, 'rb')}, params=data)

    # 打印 API 响应内容，通常包含上传结果信息
    print(r.text)




class MsgInfo(object):
    """
    消息体结构，没有想好如何使用
    """

    def __init__(self, code, msg, traceback, info_dict=None):
        self.code = code  # 状态码
        self.msg = msg
        self.traceback = traceback
        if info_dict is None:
            info_dict = {}
        self.info_dict = info_dict
        # 多个消息历史的列表
        self.msg_list = []

    def msg_lists(self):
        """
        将消息结构list存放，而不是嵌套
        :return:
        """
        self.msg_list.append({"code": self.code,
                              "msg": self.msg,
                              "traceback": self.traceback,
                              "info_dict": self.info_dict})
        return self



import asyncio
import os
import re
import sqlite3
import time
import traceback
from concurrent.futures import ProcessPoolExecutor
from pathlib import Path
from typing import List, Callable, Any

from filelock import FileLock
from tenacity import retry, stop_after_attempt, wait_exponential, wait_random, retry_if_result

from re_common.v2.baselibrary.tools.data_processer.base import BaseFileReader, BaseFileWriter


class DatabaseHandler:
    def __init__(self, db_file="processed_files.db"):
        self.db_file = db_file
        self.lock_file = f"{self.db_file}.lock"
        self._init_db()

    def _init_db(self):
        with FileLock(self.lock_file):
            """初始化 SQLite 数据库"""
            with sqlite3.connect(self.db_file) as conn:
                cursor = conn.cursor()
                cursor.execute("""
                    CREATE TABLE IF NOT EXISTS processed_files (
                        file_path TEXT PRIMARY KEY
                    )
                """)
                conn.commit()

    def save_processed_file(self, file_path):
        """保存处理过的文件"""
        with FileLock(self.lock_file):
            with sqlite3.connect(self.db_file) as conn:
                cursor = conn.cursor()
                cursor.execute(
                    "INSERT OR IGNORE INTO processed_files (file_path) VALUES (?)",
                    (file_path,)
                )
                conn.commit()

    def save_processed_files_many(self, file_paths):
        """批量保存处理过的文件路径"""
        if not file_paths:
            return
        with FileLock(self.lock_file):
            with sqlite3.connect(self.db_file) as conn:
                cursor = conn.cursor()
                cursor.executemany(
                    "INSERT OR IGNORE INTO processed_files (file_path) VALUES (?)",
                    ((fp,) for fp in file_paths)
                )
                conn.commit()

    def is_file_processed(self, file_path):
        """检查文件是否已处理"""
        with FileLock(self.lock_file):
            with sqlite3.connect(self.db_file) as conn:
                cursor = conn.cursor()
                cursor.execute(
                    "SELECT file_path FROM processed_files WHERE file_path = ?",
                    (file_path,)
                )
                result = cursor.fetchone()
            return result is not None

    def fake_processed_files(self, start_index, end_index, file_list):
        try:
            # 将字符串序号转换为整数
            start = int(start_index)
            end = int(end_index)

            # 验证序号范围
            if start >= end:
                raise ValueError(f"起始序号 {start_index} 必须小于结束序号 {end_index}")

            list_formatted_num = []
            # 为范围内的每个序号生成文件名
            for num in range(start, end):
                # 将序号格式化为5位字符串 (00120, 00121,...)
                formatted_num = f"{num:05d}"
                list_formatted_num.append(formatted_num)

            skip_path_list = []
            skip_formatted_num = []
            for file_path in file_list:
                re_f_num = re.findall(r'(?<!\d)\d{5}(?!\d)', str(Path(file_path).stem))
                if re_f_num:
                    if re_f_num[0] in list_formatted_num:
                        skip_path_list.append(file_path)
                        skip_formatted_num.append(re_f_num[0])

            for item_list in [skip_path_list[i:i + 2000] for i in range(0, len(skip_path_list), 2000)]:
                self.save_processed_files_many(item_list)
                for file_path in item_list:
                    print(f"伪造处理记录: {file_path}")

            no_fil_num = set(list_formatted_num) - set(skip_formatted_num)
            if len(no_fil_num) > 0:
                print(f"没有对应num的文件,伪造失败数量为{len(no_fil_num)},样例:{list(no_fil_num)[:10]}")
            print(f"成功伪造处理记录：序号 {start_index} 到 {end_index}（不含）的文件")

        except ValueError as e:
            print(f"错误: 序号格式无效 - {str(e)}")
        except Exception as e:
            print(f"伪造处理记录时出错: {str(e)}")


class DataProcessor:
    def __init__(
            self,
            reader: BaseFileReader,
            writer: BaseFileWriter = None,
            db_handler: DatabaseHandler = None,
            db_file="processed_files.db",
            batch_size=50,
            retry_limit=3,
    ):
        self.reader = reader
        self.writer = writer
        self.db_file = db_file
        self.batch_size = batch_size
        self.retry_limit = retry_limit
        self.db_handler = db_handler if db_handler else DatabaseHandler(db_file=db_file)

    async def retry_process_data(self, data, process_func):
        """处理数据并执行处理函数"""

        def on_retry(retry_state):
            # 每次抛错进入该函数打印消息
            print(
                f"重试次数: {retry_state.attempt_number}/{self.retry_limit},数据内容: {retry_state.args[0]}\n"
                f"异常信息: {retry_state.outcome.exception()}"
            )

        def on_retry_error(retry_state):
            # 最后抛错后调用
            original_exc = retry_state.outcome.exception()
            raise RuntimeError(
                f"处理数据失败，达到重试上限。data: {retry_state.args[0]}") from original_exc  # 抛出的自定义异常中 保留 __process_func() 里的原始错误堆栈信息（traceback）

        @retry(stop=stop_after_attempt(3),
               wait=wait_exponential(multiplier=1, min=2, max=20),
               before_sleep=on_retry,  # 每次抛错后使用
               retry_error_callback=on_retry_error,  # 如果到最后都没有成功 抛错
               reraise=True)
        async def __process_func(_data):
            return await process_func(_data)

        return await __process_func(data)

    async def process_file(self, hdfs_file_path, process_func, write_dir):
        """处理单个 gz 文件"""
        total_lines = self.reader.count_lines(hdfs_file_path)
        processed_lines = 0
        start_time = time.time()
        results = []
        #   # 这里根据不同的配置选用不同的读取文件的方法
        for lines in self.reader.read_select(hdfs_file_path):
            processing_start_time = time.time()  # 记录本批处理开始时间

            tasks = []
            for line in lines:
                # try:
                #     data = json.loads(line)
                #     tasks.append(self.retry_process_data(data, process_func))
                # except json.JSONDecodeError as e:
                #     raise Exception(f"解析JSON失败: {e}, 行内容: {line.strip()}")
                tasks.append(self.retry_process_data(line, process_func))

            # await AsyncTaskPool(self.batch_size).run(tasks) # AsyncTaskPool 适用于一次提交所有任务, 限制并发数执行
            results.extend(await asyncio.gather(*tasks))

            processed_lines += len(lines)

            elapsed_time = time.time() - start_time  # 已用时间
            processing_time = time.time() - processing_start_time  # 本次处理时间
            avg_processing_time = (
                (elapsed_time * 1000) / processed_lines if processed_lines > 0 else float("inf")
            )  # 平均每条数据的处理时间（毫秒）

            # 估算剩余时间
            remaining_time = (
                ((avg_processing_time / 1000) * (total_lines - processed_lines))
                if processed_lines > 0
                else float("inf")
            )

            # 显示总进度信息
            print(
                f"文件: {hdfs_file_path} 总进度: {processed_lines}/{total_lines} 行 | "
                f"已用时间: {elapsed_time:.2f}秒 | 本次处理时间: {processing_time:.2f}秒 | "
                f"预估剩余时间: {remaining_time:.2f}秒 | 平均每条处理时间: {avg_processing_time:.2f}毫秒"
            )

        if write_dir is not None:
            if not self.writer:
                raise Exception("没有配置写数据的对象")
            write_path = write_dir.rstrip("/") + f"/{Path(hdfs_file_path).stem}"
            self.writer.write_lines([str(item) for item in results], write_path)

        # 最终进度显示
        final_elapsed_time = time.time() - start_time  # 最终已用时间
        print(
            f"文件: {hdfs_file_path} 处理完成 | 总进度: {processed_lines}/{total_lines} 行 | "
            f"总已用时间: {final_elapsed_time:.2f}秒 | "
            f"平均每条处理时间: {(final_elapsed_time * 1000) / processed_lines:.2f}毫秒"
            if processed_lines > 0
            else "处理无数据"
        )

        self.db_handler.save_processed_file(hdfs_file_path)  # 保存处理过的文件

    async def retry_process_file(self, hdfs_file_path, process_func, write_dir):
        """带重试机制的文件处理"""

        def on_retry(retry_state):
            # 每次抛错进入该函数打印消息
            exc = retry_state.outcome.exception()
            tb = ''.join(traceback.format_exception(type(exc), exc, exc.__traceback__))
            print(tb)

            print(
                f"处理文件 {retry_state.args[0]} 时发生错误: {exc}，正在重试 {retry_state.attempt_number}/{self.retry_limit}")

        def on_retry_error(retry_state):
            # 最后抛错后调用
            print(f"处理文件 {retry_state.args[0]} 失败，达到重试上限")
            return False

        @retry(stop=stop_after_attempt(3),
               wait=wait_exponential(multiplier=1, min=2, max=20),
               before_sleep=on_retry,  # 每次抛错后使用
               retry_error_callback=on_retry_error,  # 如果最后没成功 返回 False
               reraise=True)
        async def __process_func(_hdfs_file_path, _process_func, _write_dir):
            await self.process_file(_hdfs_file_path, _process_func, _write_dir)
            return True  # 成功处理后退出

        return await __process_func(hdfs_file_path, process_func, write_dir)

    def get_file_list(self, hdfs_dir):
        # 获取所有任务文件
        all_files = self.reader.list_files(hdfs_dir)
        for file_path in all_files:
            yield file_path

    @retry(stop=stop_after_attempt(3),
           wait=wait_random(min=10, max=30),
           retry=retry_if_result(lambda result: not result),  # 如果返回值是 False（失败），则重试 最后会抛出一个默认错误tenacity.RetryError:
           reraise=True)
    async def _batch_process_file(self, hdfs_file_path: str, process_func: Callable[[str], Any],
                                  write_dir: str = None):
        """批量更新所有 gz 文件"""
        # all_succeed = True
        # for hdfs_file_path in self.get_file_list(hdfs_dir):
        #     if self.db_handler.is_file_processed(hdfs_file_path):
        #         print(f"跳过已处理文件: {hdfs_file_path}")
        #         continue  # 如果文件已处理，跳过
        #     succeed = await self.retry_process_file(hdfs_file_path, process_func, write_dir)  # 处理文件
        #     if succeed is False:
        #         all_succeed = False
        #
        # if all_succeed:
        #     # 处理完成后删除数据库文件
        #     try:
        #         if os.path.exists(self.db_file):
        #             os.remove(self.db_file)
        #             print(f"已删除断点重试文件: {self.db_file}")
        #             return True
        #     except Exception as e:
        #         print(f"删除断点重试文件失败: {e}")
        # return False
        if self.db_handler.is_file_processed(hdfs_file_path):
            print(f"跳过已处理文件: {hdfs_file_path}")
            return True  # 如果文件已处理，跳过
        succeed = await self.retry_process_file(hdfs_file_path, process_func, write_dir)  # 处理文件
        return succeed

    async def process_file_bulk(self, hdfs_file_path, process_func, write_dir):
        """按批次处理单个文件，批量数据传递给处理函数"""
        # 获取文件的数据总量
        total_lines = self.reader.count_lines(hdfs_file_path)
        processed_lines = 0
        start_time = time.time()

        results = []
        tasks = []
        # 这里根据不同的配置选用不同的读取文件的方法
        for lines in self.reader.read_select(hdfs_file_path):
            processing_start_time = time.time()  # 记录本批处理开始时间

            # batch_data = []
            # for line in lines:
            #     try:
            #         data = json.loads(line)
            #         batch_data.append(data)
            #     except json.JSONDecodeError as e:
            #         raise Exception(f"解析JSON失败: {e}, 行内容: {line.strip()}")

            # 处理读取到的批次数据
            if lines:
                tasks.append(process_func(lines))  # 将批次数据传递给处理函数并收集任务
                processed_lines += len(lines)  # 更新已处理行数

            # 当积累的任务数量达到 batch_size 时并发处理所有任务
            if len(tasks) >= self.batch_size:
                results.extend(await asyncio.gather(*tasks))
                elapsed_time = time.time() - start_time  # 已用时间
                processing_time = time.time() - processing_start_time  # 本次处理时间
                avg_processing_time = (
                    (elapsed_time * 1000) / processed_lines if processed_lines > 0 else float("inf")
                )  # 平均每条数据的处理时间（毫秒）

                # 估算剩余时间
                remaining_time = (
                    ((avg_processing_time / 1000) * (total_lines - processed_lines))
                    if processed_lines > 0
                    else float("inf")
                )

                # 显示总进度信息
                print(
                    f"文件: {hdfs_file_path} 总进度: {processed_lines}/{total_lines} 行 | "
                    f"已用时间: {elapsed_time:.2f}秒 | 本次处理时间: {processing_time:.2f}秒 | "
                    f"预估剩余时间: {remaining_time:.2f}秒 | 平均每条处理时间: {avg_processing_time:.2f}毫秒"
                )

                # 清空任务列表，准备下一批处理
                tasks.clear()
            # 处理剩余的任务
        if tasks:
            results.extend(await asyncio.gather(*tasks))  # 处理未达到 batch_size 的剩余任务

        if write_dir is not None:
            if not self.writer:
                raise Exception("没有配置写数据的对象")
            write_path = write_dir.rstrip("/") + f"/{Path(hdfs_file_path).stem}"
            self.writer.write_lines([str(item) for items in results for item in items], write_path)

        # 最终进度显示
        final_elapsed_time = time.time() - start_time  # 最终已用时间
        print(
            f"文件: {hdfs_file_path} 处理完成 | 总进度: {processed_lines}/{total_lines} 行 | "
            f"总已用时间: {final_elapsed_time:.2f}秒 | "
            f"平均每条处理时间: {(final_elapsed_time * 1000) / processed_lines:.2f}毫秒"
            if processed_lines > 0
            else "处理无数据"
        )

        self.db_handler.save_processed_file(hdfs_file_path)

    async def retry_process_file_bulk(self, hdfs_file_path, process_func, write_dir):
        """带重试机制的批量文件处理"""

        def on_retry(retry_state):
            # 每次抛错进入该函数打印消息
            exc = retry_state.outcome.exception()
            tb = ''.join(traceback.format_exception(type(exc), exc, exc.__traceback__))
            print(tb)
            print(
                f"处理文件 {retry_state.args[0]} 时发生错误: {exc}，正在重试 {retry_state.attempt_number}/{self.retry_limit}")

        def on_retry_error(retry_state):
            # 最后抛错后调用
            print(f"处理文件 {retry_state.args[0]} 失败，达到重试上限")
            return False

        @retry(stop=stop_after_attempt(3),
               wait=wait_exponential(multiplier=1, min=2, max=20),
               before_sleep=on_retry,  # 每次抛错后使用
               retry_error_callback=on_retry_error,  # 如果最后没成功 返回 False
               reraise=True)
        async def __process_func(_hdfs_file_path, _process_func, write_dir):
            await self.process_file_bulk(_hdfs_file_path, _process_func, write_dir)
            return True  # 成功处理后退出

        return await __process_func(hdfs_file_path, process_func, write_dir)

    async def batch_process_file(self, hdfs_dir: str, process_func: Callable[[List[str]], Any] | Callable[[str], Any],
                                 write_dir: str = None, is_bulk: bool = False):
        all_succeed = True
        for hdfs_file_path in self.get_file_list(hdfs_dir):
            if is_bulk:
                succeed = await self._batch_process_file_bulk(hdfs_file_path, process_func, write_dir)
            else:
                succeed = await self._batch_process_file(hdfs_file_path, process_func, write_dir)
            if succeed is False:
                all_succeed = False
        if all_succeed:
            # 处理完成后删除数据库文件
            try:
                if os.path.exists(self.db_file):
                    os.remove(self.db_file)
                    print(f"已删除断点重试文件: {self.db_file}")
                    return True
            except Exception as e:
                print(f"删除断点重试文件失败: {e}")
        return False

    @retry(stop=stop_after_attempt(3),
           wait=wait_random(min=10, max=30),
           retry=retry_if_result(lambda result: not result),  # 如果返回值是 False（失败），则重试 最后会抛出一个默认错误tenacity.RetryError:
           reraise=True)
    async def _batch_process_file_bulk(self, hdfs_file_path: str, process_func: Callable[[List[str]], Any],
                                       write_dir: str = None):
        """批量处理 gz 文件中的数据"""
        # 获取所有文件
        # all_succeed = True
        # for hdfs_file_path in self.get_file_list(hdfs_dir):
        #     # 查看是否跳过文件
        #     if self.db_handler.is_file_processed(hdfs_file_path):
        #         print(f"跳过已处理文件: {hdfs_file_path}")
        #         continue  # 跳过已处理文件
        #     # 开始批量处理文件
        #     succeed = await self.retry_process_file_bulk(hdfs_file_path, process_func, write_dir)
        #     if succeed is False:
        #         all_succeed = False
        #
        # if all_succeed:
        #     # 处理完成后删除数据库文件
        #     try:
        #         if os.path.exists(self.db_file):
        #             os.remove(self.db_file)
        #             print(f"已删除断点重试文件: {self.db_file}")
        #             return True
        #     except Exception as e:
        #         print(f"删除断点重试文件失败: {e}")
        # return False
        # 查看是否跳过文件
        if self.db_handler.is_file_processed(hdfs_file_path):
            print(f"跳过已处理文件: {hdfs_file_path}")
            return True  # 跳过已处理文件
        # 开始批量处理文件
        succeed = await self.retry_process_file_bulk(hdfs_file_path, process_func, write_dir)
        return succeed


# 全局变量，每个进程独立持有
_processor: DataProcessor | None = None
_process_func: Callable[[List[str]], Any] | Callable[[str], Any] | None = None
_process_args: dict


def get_data_processor_func(process_args):
    _func_reader = process_args["reader_func"]
    _reader_args = process_args["reader_kwargs"]
    reader = _func_reader(**_reader_args)
    writer = None
    if process_args["is_writer"]:
        _func_writer = process_args["writer_func"]
        _writer_args = process_args["writer_kwargs"]
        writer = _func_writer(**_writer_args)

    data_kwargs = {
        "reader": reader,
        "writer": writer,
        "db_file": process_args["db_file"]
    }
    if process_args.get("batch_size"):
        data_kwargs["batch_size"] = process_args["batch_size"]
    if process_args.get("retry_limit"):
        data_kwargs["retry_limit"] = process_args["retry_limit"]

    return DataProcessor(**data_kwargs)


def init_worker(process_func, process_args):
    global _processor, _process_func, _process_args
    _processor = get_data_processor_func(process_args)
    _process_func = process_func
    _process_args = process_args

    _init_func = _process_args.get("init_work", None)
    if _init_func:
        _init_func()

    _async_init_work = _process_args.get("async_init_work", None)
    if _init_func:
        asyncio.run(_async_init_work())


def worker(path_file):
    if _process_args["is_bulk"]:
        return asyncio.run(_processor._batch_process_file_bulk(path_file, _process_func, _process_args["write_dir"]))
    else:
        return asyncio.run(_processor._batch_process_file(path_file, _process_func, _process_args["write_dir"]))


def run_worker_many(hdfs_dir: str, process_func: Callable[[List[str]], Any] | Callable[[str], Any],
                    data_process_args: dict, max_workers=4):
    processor = get_data_processor_func(data_process_args)
    all_file = list(processor.get_file_list(hdfs_dir))
    with ProcessPoolExecutor(
            max_workers=max_workers,
            initializer=init_worker,
            initargs=(process_func, data_process_args)
    ) as executor:
        # 提交任务并等待结果
        results = executor.map(worker, all_file)
    # 输出结果
    for result in results:
        if result:
            print(result)



import aiohttp
from aiohttp_socks import ProxyConnector
from re_common.baselibrary.tools.all_requests.mrequest import MRequest
from re_common.baselibrary.utils.baseurl import BaseUrl
from re_common.baselibrary.utils.core.mdeprecated import aiohttp_try_except
from re_common.baselibrary.utils.core.mlamada import html_strip
from re_common.baselibrary.utils.core.requests_core import set_proxy_aio


async def default_resp_hook(self, resp):
    pass


class AioHttpRequest(MRequest):

    def __init__(self, logger=None):
        if logger is None:
            from re_common.baselibrary import MLogger
            logger = MLogger().streamlogger
        super().__init__(logger=logger)
        self.kwargs = {}
        self.client_session_kwargs = {}
        self.resp_hook = default_resp_hook
        # 预留一个字典可以向里面传入其他信息
        self.other_dicts = {}

    def set_resp_hook(self, resp_hook_func):
        self.resp_hook = resp_hook_func
        return self

    def builder(self):
        if self.refer:
            self.header["refer"] = self.refer
        self.kwargs["headers"] = self.header
        # 比如在请求之前调用API请求COOKIE,这时这个调用就没得 代理
        self.client_session_kwargs["skip_auto_headers"] = self.skip_auto_headers
        if self.proxy:
            if "socks" in self.proxy:
                if self.force_close == "true":
                    connector = ProxyConnector.from_url(self.proxy, force_close=True)
                else:
                    connector = ProxyConnector.from_url(self.proxy)
                self.client_session_kwargs["connector"] = connector
                self.sn = aiohttp.ClientSession(**self.client_session_kwargs)
            else:
                self.kwargs["proxy"] = set_proxy_aio(self.proxy)
        else:
            self.kwargs["proxy"] = set_proxy_aio(self.proxy)
        if self.force_close == "true" and "connector" not in self.client_session_kwargs.keys():
            connector = aiohttp.TCPConnector(force_close=True)
            self.client_session_kwargs["connector"] = connector
        if BaseUrl.urlScheme(self.url) == "https":
            self.kwargs["verify_ssl"] = False
        self.kwargs["timeout"] = self.timeout
        self.kwargs["allow_redirects"] = self.allow_redirects
        self.kwargs["params"] = self.params
        self.kwargs["cookies"] = self.cookies
        return self

    async def set_resp(self, resp):
        self.resp = resp
        self.set_status_code(resp.status)
        # 有时候302时我们去获取html会报错
        if self.allow_resp_text:
            if self.resp_encoding is not None:
                self.html = await resp.text(encoding=self.resp_encoding, errors=self.resp_errors)
            else:
                self.html = await resp.text(errors=self.resp_errors)
            self.html = html_strip(self.html)
        else:
            self.html = None
        if self.allow_resp_bytes:
            self.html_bytes = await resp.content.read()
        else:
            self.html_bytes = None

    @aiohttp_try_except
    async def get(self):
        if self.sn is None:
            # skip_auto_headers 用法: 不对列表内的对应参数进行自动生成
            self.sn = aiohttp.ClientSession(**self.client_session_kwargs)
        async with self.sn:
            async with self.sn.get(url=self.url, **self.kwargs) as resp:
                await self.set_resp(resp)
                await self.resp_hook(self, resp)
        return True, {"code": self.status_code, "msg": ""}

    @aiohttp_try_except
    async def post(self):
        if self.sn is None:
            self.sn = aiohttp.ClientSession(**self.client_session_kwargs)
        async with self.sn:
            async with self.sn.post(url=self.url, data=self.data, **self.kwargs) as resp:
                await self.set_resp(resp)
                await self.resp_hook(self, resp)
        return True, {"code": self.status_code, "msg": ""}

    def all_middlerwares(self, dicts):
        bools = True
        for item in self.middler_list:
            bools, dicts = item()
            if not bools:
                return bools, dicts
        return bools, dicts

    async def run(self, moths="get"):
        self.builder()
        self.on_request_start()
        if moths == MRequest.GET:
            bools, dicts = await self.get()
        elif moths == MRequest.POST:
            bools, dicts = await self.post()
        else:
            bools, dicts = False, {}
        self.on_request_end()
        if bools:
            return self.all_middlerwares(dicts)
        return bools, dicts



# 某些业务中的字符串处理 算是特定场景的工具 不算通用工具
import re

from re_common.v2.baselibrary.utils.author_smi import AuthorRatio
from re_common.v2.baselibrary.utils.string_bool import is_all_symbols
from re_common.v2.baselibrary.utils.string_clear import rel_clear
from re_common.v2.baselibrary.utils.stringutils import clean_unicode_alnum, get_alphabetic_ratio


def clean_organ_postcode(organ):
    """
    格式化组织名称字符串，移除括号内容并删除独立的6位数字（邮政编码），然后清理标点。

    备注: 该方法替换java 里面的 formatOrgan

    参数:
        organ (str): 输入的组织名称字符串，可能包含括号、分号和邮政编码。

    返回:
        str: 格式化并清理后的组织名称字符串（无独立6位数字）。
    """
    # 如果输入为空，设为空字符串以避免后续操作报错
    if not organ:
        organ = ""

    # 删除方括号和圆括号中的内容（包括括号本身）
    organ = re.sub(r"\[.*?\]", "", organ)  # 非贪婪匹配方括号内容
    organ = re.sub(r"\(.*?\)", "", organ)  # 非贪婪匹配圆括号内容

    # 定义正则表达式，匹配独立的6位数字
    # \b 表示单词边界，确保6位数字是独立的（前后不是字母、数字或下划线）
    organ = re.sub(r"\b[0-9]{6}\b", "", organ)

    # 初始化结果列表，用于存储处理后的组织名称部分
    format_organ = []
    # 按分号分割字符串，生成组织名称的各个部分
    organ_parts = organ.split(";")

    # 遍历每个部分，追加到结果列表
    for temp_organ in organ_parts:
        # 去除首尾多余空格后追加（避免因移除邮编导致的空字符串）
        cleaned_part = temp_organ.strip()
        # 如果首尾是标点符号，则移除
        # 定义标点符号的正则表达式（这里包括常见标点）
        punctuation = r"^[!,.?;:#$%^&*+-]+|[!,.?;:#$%^&*+-]+$"
        cleaned_part = re.sub(punctuation, "", cleaned_part)
        if cleaned_part:  # 只追加非空部分
            format_organ.append(cleaned_part)

    # 用分号连接结果，转换为大写并清理标点
    format_organ = ";".join(format_organ)

    # 返回最终结果并去除首尾空格
    return format_organ.strip()


def get_first_organ(organ):
    if not organ:
        return ""
    organ_list = organ.strip().split(";")
    for organ_one in organ_list:
        # 清理邮政编码
        organ_one = clean_organ_postcode(organ_one)
        if organ_one.strip():
            return organ_one

    return ""


def get_first_author(author: str) -> str:
    if not author:
        return ""
    au_list = author.strip().split(";")
    for au in au_list:
        au = re.sub("\\[.*?]", "", au)
        au = re.sub("\\(.*?\\)", "", au)
        if au.strip():
            return au
    return ""


def get_author_list(author: str):
    lists = []
    if not author:
        return []
    au_list = author.strip().split(";")
    for au in au_list:
        au = re.sub("\\[.*?]", "", au)
        au = re.sub("\\(.*?\\)", "", au)
        if au.strip():
            lists.append(au.strip())
    return lists


def get_scopus_author_abbr(author_row: str):
    if not author_row:
        return ""
    author_list = author_row.split("&&")
    if len(author_list) != 3:
        raise Exception("错误的数据个数 可能来自其他数据源")

    abbr_list = author_list[0].strip().split(";")
    abbr_list = [author.strip() for author in abbr_list if
                 author.strip() and author.strip().lower() not in ("*", "and")]
    return ";".join(abbr_list)


def get_wos_author_abbr(author_row: str):
    if not author_row:
        return ""
    author_list = author_row.split("&&")
    if len(author_list) != 4:
        raise Exception("错误的数据个数 可能来自其他数据源")
    abbr_list = []
    abbr_list_au = author_list[0].strip().split(";")
    abbr_list_ba = author_list[2].strip().split(";")
    abbr_list.extend(abbr_list_au)
    abbr_list.extend(abbr_list_ba)
    abbr_list = [author.strip() for author in abbr_list if
                 author.strip() and author.strip().lower() not in ("*", "and")]
    return ";".join(abbr_list)


def deal_rel_vol(vol_str: str):
    """
    处理 期刊融合时的卷处理逻辑
    """

    # 如果卷是全符号 清理掉
    if is_all_symbols(vol_str):
        vol_str = ""

    if vol_str.replace(".", "").isdigit():
        try:
            float_num = float(vol_str)
            if int(float_num) == float_num:
                return str(int(float_num))
        except:
            pass

    if vol_str.lower().startswith("v "):
        vol_str = vol_str.lower().replace("v ", "").strip()
        return vol_str
    if vol_str.lower().startswith("volume "):
        vol_str = vol_str.lower().replace("volume ", "").strip()
        return vol_str
    if vol_str.lower().startswith("vol. "):
        vol_str = vol_str.lower().replace("vol. ", "").strip()
        return vol_str
    if vol_str.lower().startswith("vol "):
        vol_str = vol_str.lower().replace("vol ", "").strip()
        return vol_str
    return vol_str


def deal_num_strs(input_str):
    """
    int后在str 防止有浮点型的表达方式
    """
    number_list = re.findall(r'\d+', input_str)
    transformed_numbers = [str(int(num)) for num in number_list]

    # 替换原字符串中的数字为转换后的数字
    for num, transformed_num in zip(number_list, transformed_numbers):
        input_str = input_str.replace(num, transformed_num)
    return input_str


def deal_num(num_str):
    """
    将 期格式化 方便 group尤其是有横杆的数据
    该方法 为融合二次分割时使用，如果场景合适也可以用于其他地方
    :param strs:
    :return:
    """
    # 如果期是全符号清理掉
    if is_all_symbols(num_str):
        num_str = ""

    if num_str.lower().startswith("n "):
        num_str = num_str.lower().replace("n ", "").strip()

    num_str = num_str.lower().replace("special_issue_", '').replace("_special_issue", '').replace("issue", "")
    num_str = num_str.replace("spec.", "").replace("iss.", "").replace("spl.", "").replace("special.", "").replace(
        "specialissue.", "")
    num_str = num_str.replace("spec", "").replace("iss", "").replace("spl", "").replace("special", "").replace(
        "specialissue", '')

    num_str = num_str.replace("-", "_").replace(".", "_").upper()
    num_str = num_str.lstrip("_").rstrip("_")
    if num_str.find("_") > -1:
        start, end = num_str.split("_")
        start = deal_num_strs(start)
        end = deal_num_strs(end)
        num_str = start + "_" + end
    else:
        num_str = deal_num_strs(num_str)

    return num_str.lower().strip()


def clear_author_1st(author_str:str):
    # 清理括号 防止前面流程没有清理干净
    author_str = re.sub("\\[.*?]", "", author_str)
    author_str = re.sub("\\(.*?\\)", "", author_str)

    # 清理符号
    author_str = clean_unicode_alnum(author_str)

    return author_str

def is_same_author(a1, a2):
    if get_alphabetic_ratio(a1.strip()) > 0.7 and get_alphabetic_ratio(a2.strip()) > 0.7:
        author_similar_ = AuthorRatio(a1.strip(), a2.strip())
        if author_similar_ > 0.95:
            return True
    else:
        if rel_clear(a1.strip()) == rel_clear(a2.strip()):
            return True
    return False